@@ -10,31 +10,30 @@ This script is designed to be downloaded and run independently, and
 will guide the process of downloading and installing all other source
 code.
 
-(c) 2009: Orion Buske <orion.buske@gmail.com>
+(c) 2009: Orion Buske <stasis@u.washington.edu>
 
-XXX: R and necessary R packages are not checked/installed.
-XXX: Relatively untested
 """
+MIN_SEGTOOLS_VERSION = "0.1.1"
 
 ####################### BEGIN COMMON CODE HEADER #####################
 
 import os
+import pkg_resources
 import sys
 
 from distutils.spawn import find_executable
-from distutils.version import StrictVersion
+from distutils.version import LooseVersion
 from urllib import urlretrieve
+from site import addsitedir
 from string import Template
-from subprocess import PIPE, Popen
+from subprocess import call, PIPE, Popen
+from tempfile import gettempdir
 
 assert sys.version_info >= (2, 4)
 
 MIN_HDF5_VERSION = "1.8"
 MIN_NUMPY_VERSION = "1.2"
 
-# Should match download URLs below (displayed to user)
-HDF5_DOWNLOAD_VERSION = "1.8.2"
-
 HDF5_URL = "http://www.hdfgroup.org/ftp/HDF5/prev-releases/hdf5-1.8.2/src/hdf5-1.8.2.tar.gz"
 EZ_SETUP_URL = "http://peak.telecommunity.com/dist/ez_setup.py"
 
@@ -57,24 +56,42 @@ script_dir = $scripts
 
 # One command per line
 HDF5_INSTALL_SCRIPT = """
+cd $tmpdir
 if [ ! -e $file ]; then wget $url -O $file; fi
 if [ ! -d $filebase ]; then tar -xzf $file; fi
 cd $filebase
 ./configure --prefix=$dir
 make
 make install
-rm -f $file
+cd ..
 """
 
 ####################### END COMMON CODE HEADER #####################
 
 
+import re
+
+MIN_R_VERSION = "2.8"
+
+R_URL = "http://cran.fhcrc.org/src/base/R-2/R-2.9.2.tar.gz"
+
+# List of R package pre-requisites
+R_PACKAGES = ["latticeExtra", "reshape"]
+# Repository to use for downloading R libraries through CRAN
+CRAN_REPO = "http://cran.fhcrc.org"
+
 # One command per line
-SEGTOOLS_INSTALL_SCRIPT = """
-cd $dir
-svn co $url
-cd $file
-python setup.py install
+R_INSTALL_SCRIPT = """
+cd $tmpdir
+wget $url -O $file
+if [ ! -d $filebase ]; then tar -xzf $file; fi
+cd $filebase
+./configure --prefix=$dir --enable-R-shlib
+make
+make install
+cd ..
+rm -rf $filebase
+rm -f $file
 """
 
 ####################### BEGIN COMMON CODE BODY #####################
@@ -86,6 +103,60 @@ class DependencyError(Exception):
 class InstallationError(Exception):
     pass
 
+class InteractiveShell(object):
+    """Class to manage running scripts through an interactive shell
+
+    Executes each command in a separate shell
+    """
+
+    def __init__(self, env={}):
+        self.env = dict(os.environ)
+        self.env.update(env)
+        self.old_cwd = os.getcwd()
+
+    def execute(self, command, verbose=False):
+        """Execute the given string command and return the retcode."""
+
+        if verbose:
+            print >>sys.stderr, ">> %s" % command
+
+        # Trap calls to `cd`
+        if command.strip().startswith("cd"):
+            try:
+                dest = command.strip().split()[1]
+                if dest == "-":
+                    dest = self.old_cwd
+            except IndexError:
+                dest = os.expanduser("~")  # Defaults to $HOME
+
+            # Change directory
+            self.old_cwd = os.getcwd()  # Save for `cd -`
+            os.chdir(dest)
+            return 0
+        else:
+            return call(str(command), shell=True,
+                        env=self.env, cwd=os.getcwd())
+
+    def run_script(self, script, verbose=False):
+        """Runs each string command in a list, stopping if an error occurs
+
+        verbose: if true, each command is first echo'd to stderr
+
+        """
+        for line in script:
+            if len(line) == 0:
+                continue
+
+            retcode = self.execute(line, verbose=verbose)
+            if retcode != 0:
+                raise OSError("Command failed: %s" % line)
+
+    def run_block(self, block, verbose=False):
+        """Run each line of a multi-line string as a separate command"""
+        script = block.strip().split("\n")
+        self.run_script(script, verbose=verbose)
+
+
 class ShellManager(object):
     """Class to manage details of shells.
 
@@ -102,7 +173,7 @@ class ShellManager(object):
         """Creates a ShellManager given a shell string.
 
         If shell is None, tries to use environment variable SHELL.
-        self.name: basically same as shell, but if shell is None, self.name = ""
+        self.name: basically same as shell, but if shell is None, self.name=""
         """
         self.name = shell
         if shell is None:
@@ -111,14 +182,14 @@ class ShellManager(object):
                 self.name = shell
             except KeyError:
                 self.name = ""
-                
+
         if self.name.endswith("csh"):
             self._env_format = "setenv %s %s"
         elif self.name.endswith("sh"):
             self._env_format = "export %s=%s"
         else:
             self._env_format = "set %s to %s"  # Console output
-            
+
         # What RC file are we using, or are we printing to the terminal?
         if self.name.endswith("sh"):
             self.file = os.path.join("~", ".%src" % shell)
@@ -128,14 +199,14 @@ class ShellManager(object):
                 print >>sys.stderr, "SHELL variable missing or confusing."
             else:
                 print >>sys.stderr, "Unrecognized shell: %s" % shell
-            
+
         if self.file is None:
             print >>sys.stderr, ("Shell-specific commands will be printed"
                                  " to the terminal")
-            
+
         self._out = None  # Output file not yet open
         self.has_file = (self.file is not None)
-        
+
     def set_rc_env(self, variable, value):
         """Write the given variable and value the the shell rc file (or stdout)
         """
@@ -144,10 +215,10 @@ class ShellManager(object):
                 self._out = sys.stdout
             else:
                 self._out = open(os.path.expanduser(self.file), "a")
-        
+
         cmd = self._env_format % (variable, value)
-        print >>self._out, "\n%s  # Added in segway installation\n" % cmd
-            
+        print >>self._out, "\n%s  # Added by install script" % cmd
+
     def add_to_rc_env(self, variable, value):
         """Prepend the value to the variable in the shell rc file (or stdout)
         """
@@ -196,7 +267,7 @@ def setup_arch_home():
     arch_home = prompt_user(query, default_arch_home)
     make_dir(arch_home)
     return arch_home
-            
+
 def get_default_arch_home():
     if "ARCHHOME" in os.environ:
         return os.environ["ARCHHOME"]
@@ -205,33 +276,57 @@ def get_default_arch_home():
     else:
         (sysname, nodename, release, version, machine) = os.uname()
         arch = "-".join([sysname, machine])
-        
+
     return os.path.expanduser("~/arch/%s" % arch)
 
-def setup_python_home(arch_home = None):
+def setup_python_home(arch_home=None):
     if arch_home is None:
         arch_home = get_default_arch_home()
-        
+
     query = "\nWhere should new Python packages be installed?"
     default_python_home = get_default_python_home(arch_home)
     python_home = fix_path(prompt_user(query, default_python_home))
     make_dir(python_home)
+    addsitedir(python_home)  # Load already-installed packages/eggs
+    reload(pkg_resources)  # Update working set
     return python_home, default_python_home
 
 def get_default_python_home(arch_home):
     python_version = sys.version[:3]
     return os.path.join(arch_home, "lib", "python%s" % python_version)
 
-def setup_script_home(arch_home = None):
+def setup_script_home(arch_home=None):
     if arch_home is None:
         arch_home = get_default_arch_home()
-        
+
     query = "\nWhere should new scripts and executables be installed?"
     default_script_home = os.path.join(arch_home, "bin")
     script_home = fix_path(prompt_user(query, default_script_home))
     make_dir(script_home)
     return script_home, default_script_home
 
+def setup_hdf5_installation(shell, arch_home):
+    if "HDF5_DIR" in os.environ:
+        hdf5_bin_dir = os.path.join(os.environ["HDF5_DIR"], "bin")
+        if not shell.in_env("PATH", hdf5_bin_dir):
+            # Add hdf5 bin dir to path for now to use h52gif for version
+            shell.add_to_env("PATH", hdf5_bin_dir)
+            sys.path.append(hdf5_bin_dir)
+
+    hdf5_dir = prompt_install_hdf5(arch_home)
+    if hdf5_dir:
+        print >>sys.stderr, ("\nPyTables uses the environment variable"
+                             " HDF5_DIR to locate HDF5.")
+        prompt_set_env(shell, "HDF5_DIR", hdf5_dir)
+        bin_path = os.path.join(hdf5_dir, "bin")
+        include_path = os.path.join(hdf5_dir, "include")
+        lib_path = os.path.join(hdf5_dir, "lib")
+        prompt_add_to_env(shell, "PATH", bin_path)
+        prompt_add_to_env(shell, "C_INCLUDE_PATH", include_path)
+        prompt_add_to_env(shell, "LIBRARY_PATH", lib_path)
+        prompt_add_to_env(shell, "LD_LIBRARY_PATH", lib_path)
+    return hdf5_dir
+
 def prompt_add_to_env(shell, variable, value):
     if shell.in_env(variable, value):
         print >>sys.stderr, "\nFound %s already in your %s!" % \
@@ -239,23 +334,27 @@ def prompt_add_to_env(shell, variable, value):
     else:
         shell.add_to_env(variable, value)
         query = "\nMay I edit your %s to add %s to your %s?" % \
-            (shell.file, value, variable) 
+            (shell.file, value, variable)
         permission = prompt_yes_no(query)
         if permission:
             shell.add_to_rc_env(variable, value)
-        
+
 def prompt_set_env(shell, variable, value):
-    shell.set_env(variable, value)
-    query = "\nMay I edit your %s to set your %s to %s?" % \
-        (shell.file, variable, value) 
-    permission = prompt_yes_no(query)
-    if permission:
-        shell.set_rc_env(variable, value)
+    if variable in os.environ and os.environ[variable] == value:
+        print >>sys.stderr, "\nYour %s was already set to %s!" % \
+            (variable, value)
+    else:
+        shell.set_env(variable, value)
+        query = "\nMay I edit your %s to set your %s to %s?" % \
+            (shell.file, variable, value)
+        permission = prompt_yes_no(query)
+        if permission:
+            shell.set_rc_env(variable, value)
 
 def fix_path(path):
     # Put path in standard form
     return os.path.abspath(os.path.expanduser(path))
-        
+
 def make_dir(dirname, verbose=True):
     """Make directory if it does not exist"""
     absdir = fix_path(dirname)
@@ -264,8 +363,11 @@ def make_dir(dirname, verbose=True):
         if verbose:
             print >>sys.stderr, "Created directory: %s" % dirname
 
-def substitute_template(template, fields):
-    return Template(template).substitute(fields)
+def substitute_template(template, fields, safe=False, *args, **kwargs):
+    if safe:
+        return Template(template).safe_substitute(fields, *args, **kwargs)
+    else:
+        return Template(template).substitute(fields, *args, **kwargs)
 
 def check_executable_in_path(bin):
     """Checks if an executable of the given name is in the user's path.
@@ -280,7 +382,7 @@ def check_executable_in_path(bin):
         continue_inst = prompt_yes_no(query)
         if not continue_inst:
             die("\n============ Installation Aborted ============")
-            
+
     return path
 
 def has_lsf():
@@ -315,15 +417,15 @@ def prompt_create_cfg(arch_home, python_home, default_python_home,
             write_pydistutils_cfg(cfg_path, arch_home,
                                   python_home, default_python_home,
                                   script_home, default_script_home)
-    
+
 def write_pydistutils_cfg(cfg_file, arch_home,
                           python_home, default_python_home,
                           script_home, default_script_home):
     """Write a pydistutils.cfg file
     """
     fields = {}
-    fields["prefix"] = arch_home
-    
+    fields["prefix"] = fix_path(arch_home)
+
     if python_home == default_python_home:
         platlib = "$platbase/lib/python$py_version_short"
     else:
@@ -343,16 +445,15 @@ def write_pydistutils_cfg(cfg_file, arch_home,
         print >>ofp, cfg_file_contents
     finally:
         ofp.close()
-    
-    
+
 ########################### GET VERSION ########################
 def get_hdf5_version():
     """Returns HDF5 version as string or None if not found or installed
 
-    Only works if h52gif is installed and in current user path
+    Only works if h5repack is installed and in current user path
     """
     try:
-        cmd = Popen("h52gif -V", shell=True, stdout=PIPE, stderr=PIPE)
+        cmd = Popen(["h5repack", "-V"], stdout=PIPE, stderr=PIPE)
         res = cmd.stdout.readlines()[0].strip()
         if "Version" in res:
             # HDF5 Found!
@@ -379,25 +480,48 @@ def get_setuptools_version():
         return setuptools.__version__
     except (AttributeError, ImportError):
         return None
-    
+
+def parse_download_url(url):
+    """Returns a dict of URL components"""
+    dir, filename = os.path.split(url)
+    # Remove extensions
+    filebase = filename
+    if filebase.endswith(".gz"):
+        filebase = filebase[:-3]
+    if filebase.endswith(".tar"):
+        filebase = filebase[:-4]
+    if filebase.endswith(".tgz"):
+        filebase = filebase[:-4]
+
+    assert "-" in filebase
+    filebase_tokens = filebase.split("-")
+    version = filebase_tokens[-1]
+    program = "-".join(filebase_tokens[:-1])
+    return {"dirname": dir,
+            "file": filename,
+            "filebase": filebase,
+            "version": version,
+            "program": program}
+
 def str2version(ver):  # string to version object
     if ver.startswith("$Revision:"):
         ver = ver.split()[1]  # Get revision number
-    return StrictVersion(ver)
+    return LooseVersion(ver)
 
 ##################### SPECIFIC PROGRAM INSTALLERS ################
 def prompt_install_hdf5(arch_home):
     return _installer("HDF5", install_hdf5, get_hdf5_version,
                       install_message=HDF5_INSTALL_MESSAGE, arch_home=arch_home,
-                      version=HDF5_DOWNLOAD_VERSION)
+                      url=HDF5_URL)
 
-def prompt_install_numpy():
+def prompt_install_numpy(min_version=MIN_NUMPY_VERSION):
     return _installer("Numpy", install_numpy, get_numpy_version,
-                      min_version=MIN_NUMPY_VERSION,
+                      min_version=min_version,
                       install_prompt=EASY_INSTALL_PROMPT)
 
 def install_hdf5(arch_home, *args, **kwargs):
     hdf5_dir = prompt_install_path("HDF5", arch_home)
+    make_dir(hdf5_dir)
     install_dir = install_script("HDF5", hdf5_dir, HDF5_INSTALL_SCRIPT,
                                  url=HDF5_URL)
     return install_dir
@@ -417,8 +541,8 @@ def install_numpy(min_version=MIN_NUMPY_VERSION, *args, **kwargs):
             # Make sure variable didn't return, and then replace variable
             assert "LDFLAGS" not in os.environ
             os.environ["LDFLAGS"] = env_old
-    
-#################### GENERIC INSTALL METHODS ###################         
+
+#################### GENERIC INSTALL METHODS ###################
 def _installer(progname, install_func, version_func=None,
                install_message=None, *args, **kwargs):
     """Program installation wrapper method.
@@ -427,6 +551,7 @@ def _installer(progname, install_func, version_func=None,
     progname: string name of program
     install_func: function to call with *args, **kwargs to install progname
     version_func: function to call with *args, **kwargs to find progname version
+      If version_func is none, installation will be promped no matter what.
     install_message: helpful message to print if user chooses to install program
 
     Checks if program is installed in a succificent version,
@@ -445,9 +570,7 @@ def _installer(progname, install_func, version_func=None,
                 print >>sys.stderr, "%s successfully installed." % progname
             else:
                 print >>sys.stderr, "%s not installed." % progname
-
             return success
-        
     return False
 
 def _abort_skip_install(func, *args, **kwargs):
@@ -459,7 +582,11 @@ def _abort_skip_install(func, *args, **kwargs):
     """
     try:
         return func(*args, **kwargs)
-    except InstallationError:
+    except Exception, e:
+        e_str = str(e)
+        if e_str:
+            print >>sys.stderr, "Error: %s" % e_str  # print any error message
+
         query = ("\nWould you like to try to continue the installation"
                  " without this program?")
         default = "n"
@@ -468,20 +595,27 @@ def _abort_skip_install(func, *args, **kwargs):
             return None
         else:
             die("\n============== Installation aborted =================")
-            
+
 def _check_install(progname, version_func, min_version=None, *args, **kwargs):
     """Returns True if program found with at least min_version, False otherwise
 
-    version_func should be a function that, when called, returns the version of
-    the installation as a tuple, or True if installed,
-    or None if not found/unavailable.
+    version_func should either:
+    - be a function that, when called, returns the version of
+      the installation as a tuple, or True if installed,
+      or None if not found/unavailable.
+    - or None, in which False will be returned immediately
 
     If version_func returns True, installation accepted regardless of
     min_version
+
     """
-    print >>sys.stdout, "\nSearching for %s..." % progname,
-    sys.stdout.flush()
-    version = version_func()
+    if version_func is None:
+        return False
+    else:
+        print >>sys.stdout, "\nSearching for %s..." % progname,
+        sys.stdout.flush()
+        version = version_func()
+
     if version is not None:
         print >>sys.stderr, "found!"
         if min_version is None or version is True:
@@ -492,7 +626,7 @@ def _check_install(progname, version_func, min_version=None, *args, **kwargs):
                                  " required.") % (version, min_version)
         else:
             return True
-        
+
     else:
         print >>sys.stderr, "not found."
 
@@ -517,19 +651,18 @@ def easy_install(progname, min_version=None):
                 raise InstallationError()
         else:
             raise InstallationError()
-    
-    cmd = "easy_install %s" % progname.lower()
+
+    cmd = ["easy_install", progname.lower()]
     if min_version is not None:  # Add version requirement
-        cmd += ' "%s>=%s"' % (progname, min_version)
+        cmd.append('%s>=%s' % (progname, min_version))
 
     if os.path.isdir(progname):
         print >>sys.stderr, ("\nWarning: installation may fail because"
                              " there is a subdirectory named %s at your"
                              " current path.") % progname
 
-    print >>sys.stderr, ">> %s" % cmd
-    proc = Popen(cmd, shell=True, stdout=None, stderr=None)
-    code = proc.wait()
+    print >>sys.stderr, ">> %s" % " ".join(cmd)
+    code = call(cmd, stdout=None, stderr=None)
 
     if code != 0:
         print >>sys.stderr, "Error occured installing %s" % progname
@@ -537,89 +670,69 @@ def easy_install(progname, min_version=None):
     else:
         return True
 
-def install_script(progname, prog_dir, script, **kwargs):
+def install_script(progname, prog_dir, script, safe=False, env=[], **kwargs):
     """Tries to install the specified program, given a script, url
 
     progname: string name of program being installed
-    
+
     prog_dir: directory program is to be installed in
-    
+
+    env: environment variables to be set during script execution
+
     script: multi-line string, where each line is a command to run in the
-    shell in order to install the program. Variables in this script
-    will be substituted with keywords in kwargs, as well as dir
-    (the installation directory) and, if a url is specified,
-    the downloaded file and filebase
-    
+    shell in order to install the program. Lines should be independent and
+    should use local variables not defined in the same line.
+    Variables in this script will be substituted with keywords in kwargs,
+    as well as:
+    - dir: the program installation directory
+    - file: the downloaded file (if url specified)
+    - filebase: the basename of the downloaded file (if url specified)
+    - version: the downloaded file url (if url specified and in std form)
+    - python: sys.executable (should be the python command used to call
+      this program)
+    - tmpdir: a temporary directory for downloading files to
+
     Returns installation directory if installation is successful
     (or True if unknown), and None otherwise.
+
     """
-    fields = kwargs
-
     # Set fields for template substitution
-    if "dir" not in fields:
-        fields["dir"] = prog_dir
-        
-    if "url" in fields:
-        filename = os.path.basename(fields["url"])
-        if "file" not in fields:
-            fields["file"] = filename
-        if "filebase" not in fields:
-            if filename.endswith(".tar.gz"):
-                fields["filebase"] = filename[:-7]  # minus .tar.gz
-            else:
-                fields["filebase"] = filename
-    
-    script = substitute_template(script, fields).strip().splitlines()
+    fields = {}
+    fields["dir"] = prog_dir
+    fields["python"] = sys.executable
+    fields["tmpdir"] = gettempdir()
+    if "url" in kwargs:
+        url_fields = parse_download_url(kwargs["url"])
+        fields.update(url_fields)
 
-    cwd = os.getcwd()  # Keep track of cwd state
+    # Add in kwargs (overwriting if collision)
+    fields.update(kwargs)
 
-    if prog_dir is not None:
-        os.chdir(prog_dir)  # Move to installation directory
-        
-    for line in script:
-        line = line.strip()
-        print >>sys.stderr, ">> %s" % line
-        # Intercept cd lines and update directory state
-        if line.startswith("cd"):
-            try:
-                path = line.split()[1]
-                os.chdir(fix_path(path))
-                continue
-            except IndexError:
-                die("Invalid cd command: %s" % line)
-                
-        # Run command in shell
-        proc = Popen(line, shell=True, stdout=None, stderr=None)
-        code = proc.wait()
-        if code != 0:
-            print >>sys.stderr, "Error installing: %s" % progname
-            print >>sys.stderr, "Command failed: %s" % line
-            os.chdir(cwd)
-            raise InstallationError()
+    # Make dir absolute (even if specified as kwarg)
+    fields["dir"] = fix_path(fields["dir"])
+
+    script = substitute_template(script, fields, safe=safe)
+
+    # Setup shell
+    shell = InteractiveShell(env=env)
+    shell.run_block(script, verbose=True)
 
-    os.chdir(cwd)
     return prog_dir
 
-def reinstall_program(progname, *args, **kwargs):
-    """Reinstall program given name and arguments."""
-    progname = progname.lower()
-    if progname == "hdf5":
-        install_hdf5(*args, **kwargs)
-    elif progname == "numpy":
-        install_numpy(*args, **kwargs)
-    elif progname == "segway":
-        install_segway(*args, **kwargs)
-
 ########################## PROGRAM TESTING ######################
-def prompt_test_packages(*args, **kwargs):
+def prompt_test_packages(python_home, *args, **kwargs):
     """Run each dependency's unit tests and if they fail, prompt reinstall
-    
+
     XXX: implement this for more than pytables (but numpy always fails)
+
     """
-    print >>sys.stderr, "\n\n"
+    # Start by making sure everything is up to date loaded into sys.path
+    if python_home:
+        addsitedir(python_home)
+
+    print >>sys.stderr, "\n"
     try:
         prompt_test_pytables(*args, **kwargs)
-        print >>sys.stderr, "\n=========== All tests passed! ============"
     except InstallationError:
         die("\n=========== Some tests failed! =============="
             "\nYour installation may be incomplete and might not work.")
@@ -632,26 +745,33 @@ def prompt_test_pytables(*args, **kwargs):
         try:
             import tables
             tables.test()
-            print >>sys.stderr, "Test passed!"
+            print >>sys.stderr, "Test seemed to have passed."
         except:
             print >>sys.stderr, ("There seems to be an error with the"
                                  " PyTables installation!")
             raise InstallationError()
 
 ########################## USER INTERACTION #######################
-def prompt_install(progname, install_prompt = None,
+def prompt_install(progname, install_prompt=None, url=None,
                    version=None, default="Y", *args, **kwargs):
     if version is not None:
         info = "%s %s" % (progname, version)
+    elif url is not None:
+        url_info = parse_download_url(url)
+        info = "%s %s" % (progname, url_info["version"])
     else:
         info = "%s" % progname
 
     if install_prompt is None:
         install_prompt = "May I download and install %s?"
-        
+
     query = install_prompt % info
     return prompt_yes_no(query, default=default)
 
+def prompt_path(query, default):
+    path = prompt_user(query, default)
+    return fix_path(path)
+
 def prompt_install_path(progname, default):
     query = "Where should %s be installed?" % progname
     path = prompt_user(query, default)
@@ -670,18 +790,17 @@ def prompt_yes_no(query, default="Y"):
         response = raw_input().strip().lower()
         if len(response) == 0:
             response = default.strip().lower()
-            
+
         if response.startswith("y"):
             return True
         elif response.startswith("n"):
             return False
         else:
             print >>sys.stderr, "Please enter yes or no."
-        
+
 def prompt_user(query, default=None, choices=None):
-    """Prompt user with query, given default answer and optional choices
-    """
-    
+    """Prompt user with query, given default answer and optional choices."""
+
     if choices is None:
         prompt = str(query)
     else:
@@ -693,7 +812,7 @@ def prompt_user(query, default=None, choices=None):
             prompt = "%s (%s)" % (query, " / ".join(choices))
         except (AssertionError, TypeError):
             die("Invalid choice list: %s" % choices)
-        
+
     # Loop until we get a valid response
     while True:
         # Query user and get response
@@ -730,7 +849,7 @@ def prompt_user(query, default=None, choices=None):
             else:
                 print >>sys.stderr, ("Response matched multiple choices."
                                      " Please be more specific.")
-                
+
 def die(message):
     print >>sys.stderr, str(message)
     sys.exit(1)
@@ -746,64 +865,116 @@ def main(args=sys.argv[1:]):
     except KeyError:
         shell_name = None
     shell = ShellManager(shell_name)
-                
+
     try:
-        # Set up arch_home
         arch_home = setup_arch_home()
-        
-        # Set up python home
+        prompt_set_env(shell, "ARCHHOME", fix_path(arch_home))
+
         python_home, default_python_home = setup_python_home(arch_home)
         # Add python_home to PYTHONPATH
         prompt_add_to_env(shell, "PYTHONPATH", python_home)
-        
-        # Set up bin directory
+
         script_home, default_script_home = setup_script_home(arch_home)
         # Add script_home to PATH
         prompt_add_to_env(shell, "PATH", script_home)
 
-        # Maybe create pydistutils.cfg
         prompt_create_cfg(arch_home, python_home, default_python_home,
                           script_home, default_script_home)
-            
-        # Add HDF5, if necessary
-        hdf5_dir = prompt_install_hdf5(arch_home)
-        if hdf5_dir:
-            print >>sys.stderr, ("\nPyTables uses the environment variable"
-                                 " HDF5DIR to locate HDF5.")
-            prompt_set_env(shell, "HDF5_DIR", hdf5_dir)
 
+        hdf5_dir = setup_hdf5_installation(shell, arch_home)
 
-        # Add Numpy, if necessary
         prompt_install_numpy()
 
-        # Install segtools (and dependencies)
-        prompt_install_segtools(arch_home)
+        prompt_install_R(arch_home)
 
-        # Test package installations
-        prompt_test_packages(arch_home=arch_home)
+        prompt_install_rpy2()
+
+        # Next step may need just-installed eggs, so update site list
+        addsitedir(python_home)
+
+        prompt_install_R_libs(pkgs=R_PACKAGES, repo=CRAN_REPO)
+
+        prompt_install_segtools()
+
+        # DONE: Test package installations?
+        prompt_test_packages(python_home)
 
         print >>sys.stderr, "\n=========== Installation complete ============="
-        
+        print >>sys.stderr, ("* Source your ~/.*rc file to update your"
+                             " environment *")
+
     finally:  # Clean up
         shell.close()
-    
+
 ########################### GET VERSION ########################
-def get_segtools_version():
-    """Returns segtools version as a string or None if not found or installed
-    
+def get_R_version():
+    """Returns R version as a string or None if not found or installed."""
+    try:
+        cmd = Popen(["R", "--version"], stdout=PIPE, stderr=PIPE)
+        resp = cmd.stdout.readlines()[0].strip()
+        matched = re.search("R version ([.0-9a-zA-Z]*) .*", resp)
+        if matched:
+            # R Version found! Check compilation configuration
+            print >>sys.stderr, "found!"
+            cmd = Popen(["R", "CMD", "config", "--cppflags"],
+                        stdout=PIPE, stderr=PIPE)
+            resp = cmd.stdout.readlines()[0].strip()
+            if resp == "R was not built as a library":
+                print >>sys.stderr, ("You already have R installed as a"
+                                     " program, but in order to work with"
+                                     " segtools, it must be installed as"
+                                     " a library.\nR library"),
+                return None
+            else:
+                print >>sys.stderr, "Searching for R library..."
+                return matched.group(1)
+        else:
+            return None
+    except (OSError, IndexError):
+        return None
+
+def get_rpy2_version():
+    """Returns rpy2 version as a string or None if not found or installed
+
     Temporarily removes '.' from sys.path during installation to prevent
-    finding segtools in current directory (but uninstalled)
+    finding rpy2 in current directory (but uninstalled)
     """
     dir = os.getcwd()
     index = None
     if dir in sys.path:
         index = sys.path.index(dir)
         del sys.path[index]
-        
+
     try:
         try:
-            import segtools
-            return segtools.__version__
+            import rpy2
+            return rpy2.__version__
+        except (AttributeError, ImportError):
+            return None
+    finally:
+        if index is not None:
+            sys.path.insert(index, dir)
+
+def get_segtools_version():
+    """Returns segtools version as a string or None if not found or installed
+
+    Temporarily removes '.' from sys.path during installation to prevent
+    finding segtools in current directory (but uninstalled).
+
+    Since segtools __version__ is currently a revision number, get the full
+    number from pkg_resources
+    """
+    dir = os.getcwd()
+    index = None
+    if dir in sys.path:
+        index = sys.path.index(dir)
+        del sys.path[index]
+
+    try:
+        try:
+            ref = pkg_resources.Requirement.parse("segtools")
+            data = pkg_resources.working_set.find(ref)
+            return data.version
         except (AttributeError, ImportError):
             return None
     finally:
@@ -811,16 +982,78 @@ def get_segtools_version():
             sys.path.insert(index, dir)
 
 ##################### SPECIFIC PROGRAM INSTALLERS ################
-def prompt_install_segtools(arch_home):
+def prompt_install_rpy2():
+    return _installer("rpy2", install_rpy2, get_rpy2_version,
+                      install_prompt=EASY_INSTALL_PROMPT)
+
+def prompt_install_R(arch_home, min_version=MIN_R_VERSION):
+    return _installer("R", install_R, get_R_version, url=R_URL,
+                      min_version=min_version, arch_home=arch_home)
+
+def prompt_install_R_libs(pkgs=R_PACKAGES, repo=CRAN_REPO):
+    query = "\nMay I download and install necessary R libraries?"
+    permission = prompt_yes_no(query)
+    if permission:
+        return install_R_libs(pkgs=pkgs, repo=repo)
+    else:
+        return False
+
+def prompt_install_segtools(min_version=MIN_SEGTOOLS_VERSION):
     return _installer("segtools", install_segtools, get_segtools_version,
-                      install_prompt = EASY_INSTALL_PROMPT, arch_home=arch_home)
-         
-def install_segtools(arch_home, *args, **kwargs):
-    segtools_dir = prompt_install_path("segtools", arch_home)
-    install_dir = install_script("segtools", segtools_dir,
-                                 SEGTOOLS_INSTALL_SCRIPT, url=SEGTOOLS_URL)
-    return install_dir
+                      install_prompt=EASY_INSTALL_PROMPT,
+                      min_version=min_version)
+
+def install_R(arch_home, *args, **kwargs):
+    """Install R to arch_home.
+
+    Set R_PROFILE_USER to /dev/null for the installation,
+    else, ~/.Rprofile code might crash, causing installation to fail/
+
+    """
+    if "R_PROFILE_USER" in os.environ:
+        old = os.environ["R_PROFILE_USER"]
+        os.environ["R_PROFILE_USER"] = "/dev/null"
+    else:
+        old = None
+
+    try:
+        install_dir = install_script("R", arch_home, R_INSTALL_SCRIPT,
+                                     url=R_URL)
+        return install_dir
+    finally:
+        if old is not None:
+            os.environ["R_PROFILE_USER"] = old
+
+def install_rpy2(*args, **kwargs):
+    return easy_install("rpy2")
+
+def install_R_libs(pkgs=R_PACKAGES, repo=CRAN_REPO, *args, **kwargs):
+    """Install R libs using CRAN.
+
+    Temporary unsets DISPLAY to try to not open X-window for ssh-ers.
+
+    """
+    if "DISPLAY" in os.environ:
+        old = os.environ["DISPLAY"]
+        del os.environ["DISPLAY"]
+    else:
+        old = None
+    try:
+        try:
+            from rpy2.robjects import r, numpy2ri
+            # numpy2ri imported for side-effects
+            from numpy import array
+
+            r["install.packages"](array(pkgs), dep=True)
+        except ImportError:
+            raise InstallationError("rpy2 required to install R libs!")
+    finally:
+        if old is not None:
+            os.environ["DISPLAY"] = old
+
+def install_segtools(min_version=MIN_SEGTOOLS_VERSION, *args, **kwargs):
+    return easy_install("segtools", min_version=min_version)
+
 
-            
 if __name__ == "__main__":
     sys.exit(main())
