@@ -9,22 +9,26 @@ lattice.options(default.theme = "theme.dark2",
 
 # Generate log labels
 at.log <- function(low.exp = 0, high.exp = 9) {
-  10^seq(from=low.exp, to=high.exp)
+  10^seq(from = low.exp, to = high.exp)
 }
 
 
 ## Generate axis label list for a reasonable axis
-at.pretty <- function(from=0, to=10, length=4, largest=FALSE) {
+at.pretty <- function(from = 0, to = 10, length = 4,
+                      largest = FALSE, trim = TRUE) {
   if (from == to) {
     list(from)
   } else {
-    # Trim sequence from 0-end
-    trim <- if (abs(from) < abs(to)) head else tail
-    at.seq <- trim(pretty(c(from, to), n=length, min.n=length), n=length)
-    trim <- if (abs(from) < abs(to)) tail else head
+    at.seq <- pretty(c(from, to), n = length, min.n = length)
+    if (trim) {
+      # Trim sequence from 0-end
+      trimmer <- if (abs(from) < abs(to)) head else tail
+      at.seq <- trimmer(at.seq, n = length)
+    }
+    trimmer <- if (abs(from) < abs(to)) tail else head
     if (largest) {
       # Get only largest value (by abs)
-      trim(at.seq, n=1)
+      trimmer(at.seq, n = 1)
     } else {
       at.seq
     }
@@ -44,6 +48,12 @@ labels.log <- function(...) {
   sapply(at.log(...), label.log)
 }
 
+read.mnemonics <- function(filename, stringsAsFactors = NULL, colClasses = NULL,
+                           ...) {
+  read.delim(filename, stringsAsFactors = FALSE, colClasses = "character",
+             ...)[, 1:2]
+}
+
 # Given a list of labels (e.g. levels(data$label)), returns a dataframe
 # with group and index fields, corresponding to the character and numeric 
 # components of each label.
@@ -59,14 +69,13 @@ labels.classify <-
   for (i in 1:length(labels.parts)) {
     label <- labels.parts[[i]]
     if (length(label) > 1) {
-      # Splitting worked!
+      ## Splitting worked!
       labels.split[i, 1:2] <- label[1:2]
     } else {
-      # Splitting didn't work. Try treating like normal mnemonics (e.g. TF0)
+      ## Splitting didn't work. Try treating like normal mnemonics (e.g. TF0)
       match <- regexpr("[0-9]+$", label)
       match.len <- attributes(match)$match.length
-      matched <- (match.len > 0)
-      if (matched) {
+      if (match.len > 0) {
         labels.split[i, 1] <- substring(label, 1, match - 1)
         labels.split[i, 2] <- substring(label, match, nchar(label))
       } else {
@@ -107,7 +116,8 @@ relabel.factor <- function(field, mnemonics) {
 smart.factor.reorder <- function(field) {
   levels.raw <- levels(field)
   suppressWarnings(levels.int <- as.integer(levels.raw))
-  levels.valid <- is.finite(levels.int)
+  levels.valid <- (is.finite(levels.int) &
+                   as.character(levels.int) == as.character(levels.raw))
   levels.full <- c(sort(levels.int[levels.valid]), 
                    sort(levels.raw[!levels.valid]))
 
@@ -153,15 +163,15 @@ theme.slide <- function() {
  res <- theme.dark2()
 
  additions <-
-   list(axis.text = list(cex = 1.5),
+   list(axis.text = list(cex = 1.3),
         par.main.text = list(cex = 2),
         par.sub.text = list(cex = 2),
         layout.heights = list(axis.top = 1.5, axis.bottom = 1.25),
         layout.widths = list(axis.left = 1.25),
         add.text = list(cex = 1.5),
-        par.xlab.text = list(cex = 2),
-        par.ylab.text = list(cex = 2),
-        par.zlab.text = list(cex = 2))
+        par.xlab.text = list(cex = 1.5),
+        par.ylab.text = list(cex = 1.5),
+        par.zlab.text = list(cex = 1.5))
 
  modifyList(res, additions)
 }
@@ -264,7 +274,7 @@ dev.print.images <- function(basename, dirname,
     filename.pdf <-
       save.image(basename, "pdf", dirname, device.pdf,
                  width = width.pdf, height = height.pdf,
-                 useDingbats = FALSE, ...)
+                 useDingbats = FALSE, as.slide = TRUE, ...)
   }
 
   if (make.thumb) {
