@@ -8,10 +8,11 @@ library(reshape)
 ############### OVERLAP ANALYSIS ##############
 
 read.overlap <- function(filename, mnemonics = NULL, ...,
-                         check.names = FALSE)
+                         colClasses = list(X = "character"))  # label column
 {
-  res <- read.delim(filename, ..., check.names = check.names)
-  colnames(res)[1] = "label"
+  res <- read.delim(filename, ..., colClasses = colClasses)
+  colnames(res)[1] <- "label"
+  # Order by file order
   res$label <- factor(res$label, levels = unique(res$label))
 
   if (length(mnemonics) > 0) {
@@ -155,10 +156,10 @@ write.stats <-
 xyplot.overlap <-
   function(x = tpr ~ fpr | factor, 
            data,
-           small.cex = 2.0,
-           large.cex = 2.4,
+           small.cex = 1.8,
+           large.cex = 2.0,
            as.table = TRUE,
-           aspect = "fill",
+           aspect = "iso",
            auto.key = FALSE, #list(space = "right"),
            xlab = list("False positive rate (FP / (FP + TN))", cex = large.cex),
            ylab = list("True positive rate (TP / (TP + FN))", cex = large.cex),
@@ -206,9 +207,10 @@ plot.overlap <-
 
 read.pvalues <- 
   function(filename, mnemonics = NULL,
-    ..., header = TRUE, check.names = FALSE)
+           ..., header = TRUE,
+           colClasses = list(X = "character")) # Label column
 {
-  res <- read.delim(filename, ..., header = header, check.names = check.names)
+  res <- read.delim(filename, ..., header = header, colClasses = colClasses)
   colnames(res)[1] = "label"
 
   # Order label factor by order in label col
@@ -224,55 +226,61 @@ read.pvalues <-
 }
 
 panel.pvalues <-
-  function(x, y, groups, subscripts, reference, ...)
+  function(x, y, reference = 0.01, colors = NULL, ...)
 {
-  # Plot p-value == 0.01
-  panel.abline(c(0.01, 0), reference = TRUE, ...)
 
-  panel.levelplot(x, y, groups = groups, subscripts = subscripts, ...)
+  ref.x <- log10(reference)
+  panel.refline(v = ref.x)
+  #panel.axis(side = "bottom", at = c(ref.x), outside = TRUE,
+  #           labels = c(paste("p = ", as.character(reference))))
+  special = is.infinite(x)
+  if (!is.null(colors)) {
+    col = colors[special]
+  } else {
+    col = NA
+  }
+  x[special] <- min(x[!special]) * 2
+  #panel.abline(x = c(-10, 1), h = y[special], col = col, lwd = 2, lty = 3, ...)
+  
+  panel.barchart(x, y, stacked = FALSE, col = colors, ...)
 }
 
-levelplot.pvalues <-
-  function(x = log10(value) ~ variable + label,
-           data, 
-           min.val = 1e-6,
-           scales = list(x = list(rot = 90), 
-                         z = list(log = TRUE)),
-           aspect = "iso",
+barchart.pvalues <-
+  function(data,
+           x = reorder(label, -value) ~ value,
            as.table = TRUE,
-           main = "Overlap significance [log10(p-value)]",
+           main = "Approximate significance of overlap",
            panel = panel.pvalues,
-           xlab = "Feature class",
+           xlab = "p-value",
            ylab = "Segment label",
-           ..., 
-           palette = colorRampPalette(rev(brewer.pal(11, "RdBu")),
-             interpolate = "spline", space = "Lab")(100))
+           reference = 0.01,
+           origin = 0,
+           auto.key = if (nlevels(data$variable) > 1) { TRUE } else { FALSE },
+           colors = label.colors(data$label),
+           scales = list(x = list(log = TRUE)),
+           ...)
 {
   data.melted <- melt(data, id.vars="label")
-
-  # Reverse label ordering for plotting
-  data.melted$label <- factor(data.melted$label, 
-                              levels=rev(levels(data.melted$label)))
-
-  # Truncate small values
-  data.melted$value[data.melted$value < min.val] <- min.val
-
-  levelplot(x = x,
-            data = data.melted,
-            scales = scales,
-            aspect = aspect,
-            as.table = as.table,
-            cuts = 99,
-            col.regions = palette,
-            main = main,
-            xlab = xlab,
-            ylab = ylab)
+  xyplot(x = x,
+         data = data.melted,
+         group = variable,
+         panel = panel,
+         as.table = as.table,
+         reference = reference,
+         main = main,
+         colors = colors,
+         scales = scales,
+         origin = origin,
+         auto.key = auto.key,
+         xlab = xlab,
+         ylab = ylab,
+         #par.settings = list(clip = list(panel = "off")),
+         ...)
 }
 
 plot.overlap.pvalues <- 
   function(tabfile, mnemonics = NULL, ...)
 {
   pvalues <- read.pvalues(tabfile, mnemonics = mnemonics)
- 
-  levelplot.pvalues(data = pvalues, ...)
-}
\ No newline at end of file
+  barchart.pvalues(pvalues, ...)
+}
