@@ -1,6 +1,6 @@
 from __future__ import division, with_statement
 
-__version__ = "$Revision: 90 $"
+__version__ = "$Revision: 151 $"
 
 """
 common.py:
@@ -20,7 +20,8 @@ from contextlib import closing, contextmanager
 from functools import partial
 from genomedata import Genome
 from gzip import open as _gzip_open
-from numpy import array, empty, iinfo
+from numpy import array, concatenate, empty, iinfo
+from operator import itemgetter
 from pkg_resources import resource_filename, resource_string
 from string import Template
 from tabdelim import DictReader, DictWriter
@@ -34,7 +35,7 @@ except NameError:
 
 PKG_R = os.extsep.join([PKG, "R"])
 PKG_RESOURCE = os.extsep.join([PKG, "resources"])
-    
+
 from .bed import read_native
 
 EXT_GZ = "gz"
@@ -79,9 +80,9 @@ class Segmentation(object):
     labels: a dict
       key: int ("label_key")  (a unique id)
       val: printable (str or int)  (what's in the actual BED file)
-      
+
     tracks: a list of track names that were used to obtain the segmentation
-    segtool: the tool that was used to obtain this segmentation (e.g. segway)  
+    segtool: the tool that was used to obtain this segmentation (e.g. segway)
     """
 
     def __init__(self, chromosomes, labels, tracks, segtool):
@@ -107,29 +108,29 @@ class OutputMasker:
         pass  # Mask output
     def restore(self):
         return self._stream
-        
+
 ## Wrapper for gff/gtf feature data
 # class Feature(object):
 #     def __init__(self, line=None, tokens=None):
 #         assert (line or tokens) and not (line and tokens)
 #         if tokens is None:
 #             tokens = line.strip().split("\t")
-            
+
 #         self.__dict__ = dict(zip(FEATURE_FIELDS, tokens))
-            
+
 #         # Make zero-based, exclusive end:
 #         #   http://genome.ucsc.edu/FAQ/FAQformat#format3
 #         self.start = int(self.start) - 1
 #         self.end = int(self.end)
-        
-        
+
+
 ## UTILITY FUNCTIONS
-        
+
 ## Die with error message
 def die(msg="Unexpected error"):
     print >> sys.stderr, "\nERROR: %s\n" % msg
     sys.exit(1)
-        
+
 def make_filename(dirpath, basename, ext):
     return os.path.join(dirpath, os.extsep.join([basename, ext]))
 
@@ -151,8 +152,9 @@ def make_id(modulename, dirpath):
 
 def check_clobber(filename, clobber):
     if (not clobber) and os.path.isfile(filename):
-        die("Output file: %s already exists! Use --clobber to overwrite!" % filename)
-        
+        die("Output file: %s already exists! Use --clobber to overwrite!" % \
+                filename)
+
 def gzip_open(*args, **kwargs):
     return closing(_gzip_open(*args, **kwargs))
 
@@ -161,7 +163,7 @@ def maybe_gzip_open(filename, *args, **kwargs):
         return gzip_open(filename, *args, **kwargs)
     else:
         return open(filename, *args, **kwargs)
-    
+
 def fill_array(scalar, shape, dtype=None, *args, **kwargs):
     if dtype is None:
         dtype = array(scalar).dtype
@@ -190,7 +192,7 @@ def template_substitute(filename):
     Simplify import resource strings in the package
     """
     return Template(template_string(filename)).substitute
-        
+
 @contextmanager
 def tab_saver(dirpath, basename, fieldnames, comment=None,
               clobber=False, header=True, verbose=True):
@@ -199,7 +201,7 @@ def tab_saver(dirpath, basename, fieldnames, comment=None,
     """
     if verbose:
         print >>sys.stderr, "Saving tab file...",
-        
+
     outfilename = make_tabfilename(dirpath, basename)
     check_clobber(outfilename, clobber)
     with open(outfilename, "w") as outfile:
@@ -211,61 +213,13 @@ def tab_saver(dirpath, basename, fieldnames, comment=None,
                 print >> outfile, "# %s" % comment
         yield DictWriter(outfile, fieldnames,
                          extrasaction="ignore", header=header)
-        
+
     if verbose:
         print >>sys.stderr, "done"
 
-## OLD IMAGE SAVER
-## Generator to save an R plot to both png and pdf with only one plot
-## Yields to caller to plot, then saves images
-# @contextmanager
-# def image_saver(dirpath, basename, clobber=False, **kwargs):
-#     png_filename = make_filename(dirpath, basename, EXT_PNG)
-#     png_slide_filename = make_filename(dirpath,
-#                                        "%s%s" % (basename, ".slide"),
-#                                        EXT_PNG)
-#     pdf_filename = make_filename(dirpath, basename, EXT_PDF)
-
-#     # Make sure there is no silent overwrite
-#     filenames = [png_filename, png_slide_filename, pdf_filename]
-#     if not clobber:
-#         file_exists = list(os.path.isfile(file) for file in filenames)
-#         if any(file_exists):
-#             die("Output file: %s already exists! Use --clobber to overwrite!" % filenames[file_exists.index(True)])
-        
-#     # Set up primary device
-#     try:
-#         r.png(png_filename, **kwargs)
-#     except rinterface.RRuntimeError:
-#         die('If unable to start PNG device, try `export DISPLAY=":1"` from the shell before re-running validation')
-        
-#     png_device = r['dev.cur']()
-        
-#     r['dev.control']("enable")  # Allow copying from device
-
-#     print >>sys.stderr, "\tCreating images..."
-#     yield  # Wait for plot
-
-#     try:
-#         # Make pdf copy
-#         # Dingbats off because otherwise circles displayed as q's
-#         r['dev.copy2pdf'](file=pdf_filename, useDingbats=False)
-        
-#         # Make slide copy
-#         r['dev.print.slide'](png_slide_filename)
-        
-#         # Flush buffer and close plot
-#         r['dev.off'](png_device)
-        
-#         print >>sys.stderr, "\tImages saved!"
-#     except:
-#         r['dev.off'](png_device)
-#         print >> sys.stderr, "ERROR: R crashed. Some images might not be saved."
-
-## CURRENT VERSION OF IMAGE_SAVER
 @contextmanager
-def image_saver(dirpath, basename, clobber = False, verbose = True,
-                downsample = False, **kwargs):
+def image_saver(dirpath, basename, clobber=False, verbose=True,
+                downsample=False, **kwargs):
     """
     Generator to save an R plot to both png and pdf with only one plot
     Yields to caller to plot, then saves images
@@ -278,21 +232,23 @@ def image_saver(dirpath, basename, clobber = False, verbose = True,
 
     if verbose:
         print >>sys.stderr, "Creating images...",
-        
+
     try:
         png_filename = make_pngfilename(dirpath, basename)
         r.png(png_filename, **kwargs)
         png_device = r["dev.cur"]()
         r["dev.control"]("enable")
     except rinterface.RRuntimeError:
-        die('Image creation failed.\nIf unable to start PNG device, try setting (export/setenv) variable DISPLAY to ":1" from the (bash/c) shell before re-running validation.')
-        
+        die('Image creation failed.\nIf unable to start PNG device, try'
+            ' setting (export/setenv) variable DISPLAY to ":1" from the'
+            ' (bash/c) shell before re-running validation.')
+
     yield  # Wait for plot
-    
+
     # Use R routine to create all the other images (pdf, slide, etc)
     try:
-        r["dev.print.images"](basename = basename, dirname = dirpath,
-                              downsample = downsample, **kwargs)
+        r["dev.print.images"](basename=basename, dirname=dirpath,
+                              downsample=downsample, **kwargs)
         r["dev.off"](png_device)
     except rinterface.RRuntimeError:
         print >> sys.stderr, 'ERROR: Images might not have been saved!'
@@ -321,15 +277,15 @@ def parse_bed_header(line):
 ## Maps label_keys over segments
 def map_segments(segments, labels, chrom_size):
     """
-    converts a segment mapping into label_keys at every nucleotide position 
+    converts a segment mapping into label_keys at every nucleotide position
         in the chromosome
-        
-    e.g.  segment labeled 0 at position [0,4) followed by 
+
+    e.g.  segment labeled 0 at position [0,4) followed by
           segment labeled 1 at position [4,7) gets converted into:
           0000111
-    """  
+    """
     segments_dtype = segments.dtype  # MA: the data type of segments
-    segments_dtype_max = iinfo(segments_dtype).max # sentinel value		
+    segments_dtype_max = iinfo(segments_dtype).max  # sentinel value
     # MA: the maximum value supported by the type
     assert segments_dtype_max not in labels.keys()
 
@@ -351,7 +307,7 @@ def loop_segments_continuous(chromosome, segmentation, verbose=True):
         if verbose:
             print >>sys.stderr, "\t\tskipping: no data"
         raise StopIteration
-    
+
     supercontig_iter = chromosome.itercontinuous()
     supercontig = None
     supercontig_last_start = 0
@@ -375,17 +331,19 @@ def loop_segments_continuous(chromosome, segmentation, verbose=True):
                                              supercontig.end)
 
         if verbose and segment_i % 100 == 0:
-            print >>sys.stderr, "\r\t\t\tSegment %d / %d" % (segment_i, num_segments),
+            print >>sys.stderr, "\r\t\t\tSegment %d / %d" % \
+                (segment_i, num_segments),
             sys.stdout.flush()
-            
+
         if seg_end <= supercontig.start:
             continue  # Get next segment
 
         yield segment, continuous[seg_start:seg_end]
 
     if verbose:
-        print >>sys.stderr, "\r\t\t\tSegment %d / %d" % (segment_i, num_segments)
-        
+        print >>sys.stderr, "\r\t\t\tSegment %d / %d" % \
+            (segment_i, num_segments)
+
 
 ## Returns splice of continuous or sequece from supercontig within segment
 ## range. Datatype should be "continuous" or "sequence".
@@ -394,18 +352,19 @@ def loop_segments_continuous(chromosome, segmentation, verbose=True):
 def get_supercontig_splice(chromosome, start, end, datatype="continuous"):
     if datatype not in ("continuous", "sequence"):
         die("Unknown data type in 'get_supercontig_splice!")
-        
+
     # iterate through each supercontig
     saved = array([])
     for supercontig, continuous in chromosome.itercontinuous():
-        #print >>sys.stderr, "Examining supercontig [%d, %d]" % (supercontig.start, supercontig.end)
+        #print >>sys.stderr, "Examining supercontig [%d, %d]" % \
+        #    (supercontig.start, supercontig.end)
 
         # Select appropriate data stream
         if datatype == "continuous":
             data = continuous
         else:
             data = supercontig.seq
-            
+
         # XXX Make sure there are no OBOBS in indexing!
         if start >= supercontig.start and end <= supercontig.end:
             # Found supercontig that completely covers range
@@ -437,7 +396,8 @@ def setup_directory(dirname):
     else:
         # Require write and execute permissions on existing dir
         if not os.access(dirname, os.W_OK | os.X_OK):
-            die("Error: Output directory does not have appropriate permissions!")
+            die("Error: Output directory does not have appropriate"
+                " permissions!")
 
 ## Given labels and mnemonics, returns an ordered list of label_keys
 ##   and a new labels dict mapping label_keys to label strings
@@ -467,9 +427,9 @@ def get_ordered_labels(labels, mnemonics=[]):
                 partial_int_labels[label] = key
         ordered_keys = list(partial_int_labels[key]
                             for key in sorted(partial_int_labels.keys()))
-        
+
     return ordered_keys, labels
-    
+
 ## Maps the provided labels to mnemonics (or some other mnemonic file
 ##   field specified ready to be fed into R.
 ## Returns a numpy.array of strings with a row of [oldlabel, newlabel] for
@@ -478,7 +438,7 @@ def get_ordered_labels(labels, mnemonics=[]):
 def map_mnemonics(labels, mnemonicfilename, field="mnemonic"):
     if mnemonicfilename is None:
         return []
-    
+
     label_order, label_data = load_mnemonics(mnemonicfilename)
     str_labels = labels.values()
 
@@ -497,7 +457,7 @@ def map_mnemonics(labels, mnemonicfilename, field="mnemonic"):
             mnemonics.append([label, label])
 
     return array(mnemonics)
-            
+
 ## Loads segmentation label descriptions and mnemonics
 ##   from a tab-delimited file with a header row
 def load_mnemonics(filename):
@@ -506,10 +466,10 @@ def load_mnemonics(filename):
                index    description    mnemonic
       e.g.     4    initiation (strong)    IS
     Returns a tuple of (label_order, label_data)
-    
+
     label_order: an ordered list of string label indices,
       corresponding to the preferred display order in plots
-      
+
     label_data: dict
       key: a string label index
       value: a dict with fields including "mnemonic" and "description",
@@ -520,7 +480,7 @@ def load_mnemonics(filename):
         return []
     elif not os.path.isfile(filename):
         die("Could not find mnemonic file: %s" % filename)
-        
+
     label_order = []
     label_data = {}
     with open(filename, "rU") as ifp:
@@ -529,63 +489,71 @@ def load_mnemonics(filename):
             label_index = row["index"]
             label_order.append(label_index)
             label_data[label_index] = row
-    
+
     return (label_order, label_data)
 
-def key_by_field(fieldname):
-    def entry_key(entry):
-        return entry[fieldname]
-    return entry_key
-
 ## Parses gff file for features
-def load_gff_data(gff_filename):
+def load_gff_data(gff_filename, sort=True):
     '''
     Expects data in GFF format (1-indexed locations):
     CHROM<tab>source<tab>feature<tab>START<tab>END<tab>score<tab>STRAND
     chrom, start, end, and strand are required
     strand can be '+'/'-' or '.', but not both in the same file
 
-    Returns tssdata
-    tssdata: a dict
+    File may be gzip'd, but if so, must have .gz as the extension
+
+    Returns gffdata
+    gffdata: a dict
       key: string chromosome name (e.g. "chr13")
       val: a list of dicts (ordered by the start index, ascending)
-        key: "start", "end", or "strand"
+        key: "start", "end", "strand", "source"
         val: the associated data item, if it exists
              string for strand
              int (zero-indexed) for start and end (exclusive)
     '''
     data = defaultdict(list)
     stranded = None
-    with open(gff_filename) as infile:
+    with maybe_gzip_open(gff_filename) as infile:
         for line in infile:
-            tokens = line.strip().split("\t")
-            chrom = tokens[0]
-
-            name = tokens[2]
-            start = int(tokens[3]) - 1  # Make zero-indexed
-            # Make zero-indexed and exclusive (these cancel out)
-            end = int(tokens[4])
+            # Ignore comments
+            if line.startswith("#"): continue
 
+            # Parse tokens from GFF line
             try:
-                strand = tokens[6]
-            except IndexError:
-                strand = "."
-                
-            if strand == "+" or strand == "-":
-                assert stranded or stranded is None
-                stranded = True
-            else:
-                assert not stranded  # Don't have both +/- and .
-                strand = "."  # N/A
-                stranded = False
-                
-            feature = dict(name=name, start=start, end=end, strand=strand)
-            data[chrom].append(feature)
-            
-    # Sort features by ascending start        
-    for chrom_features in data.itervalues():
-        chrom_features.sort(key=key_by_field("start"))
-        
+                fields = {}
+                tokens = line.strip().split("\t")
+
+                chrom = tokens[0]
+                fields["source"] = tokens[1]
+                fields["name"] = tokens[2]
+                fields["start"] = int(tokens[3]) - 1  # Make zero-indexed
+                # Make zero-indexed and exclusive (these cancel out)
+                fields["end"] = int(tokens[4])
+
+                try:
+                    strand = tokens[6]
+                except IndexError:
+                    strand = "."
+
+                if strand == "+" or strand == "-":
+                    assert stranded or stranded is None
+                    stranded = True
+                else:
+                    assert not stranded  # Don't have both +/- and .
+                    strand = "."  # N/A
+                    stranded = False
+
+                fields["strand"] = strand
+
+                data[chrom].append(fields)
+            except (IndexError, ValueError):
+                die("Error parsing fields from feature line:\n\t%s" % line)
+
+    if sort:
+        # Sort features by ascending start
+        for chrom_features in data.itervalues():
+            chrom_features.sort(key=itemgetter("start"))
+
     return data
 
 def load_segmentation(filename, checknames=True):
@@ -598,10 +566,10 @@ def load_segmentation(filename, checknames=True):
     If there are some integer and some string labels, an error will be raised
       (unless checknames is False).
     """
-    
+
     print >>sys.stderr, "Loading segmentation...",
     sys.stdout.flush()
-    
+
     # first get the tracks that were used for this segmentation
     segtool, tracks = get_bed_metadata(filename)
 
@@ -620,25 +588,26 @@ def load_segmentation(filename, checknames=True):
                     except ValueError:
                         checknames = False
                         if len(label_dict) > 0:  # Previous int labels found
-                            die("Found both integer and string labels in BED file: %s" % filename)
+                            die("Found both integer and string labels in BED"
+                                "file: %s" % filename)
                 if not checknames:  # Sequential if's important
                     label_key = len(label_dict)
-                    
+
                 label_dict[label] = label_key
-                
+
             segment = (datum.chromStart, datum.chromEnd, label_key)
             data[datum.chrom].append(segment)
 
     # Create reverse dict for return
     labels = dict((val, key) for key, val in label_dict.iteritems())
-    print >>sys.stderr, ""
+    print >>sys.stderr, "\nMapping names to integers"
     for key, label in labels.iteritems():
-        print >>sys.stderr, "%d -> \"%s\"" % (key, label)
-            
+        print >>sys.stderr, "\"%s\" -> %d" % (label, key)
+
     # Sort segments within each chromosome by start index
     for segments in data.itervalues():
-        segments.sort()
-        
+        segments.sort(key=itemgetter(0))
+
     # convert lists of tuples to array
     chromosomes = dict((chrom, array(segments))
                        for chrom, segments in data.iteritems())
@@ -652,7 +621,8 @@ def load_genome(genomedatadir):
     if genomedatadir is not None and os.path.isdir(genomedatadir):
         genome = Genome(genomedatadir)
         if not genome or genome is None:
-            die("Error: Unable to load genome data from directory: %s" % genomedatadir)
+            die("Error: Unable to load genome data from directory: %s" % \
+                    genomedatadir)
         else:
             return genome
     else:
