@@ -17,17 +17,19 @@ as provided by exporting UCSC gene data in GFF format.
 # Must be the same as the folder containing this script
 MODULE="feature_aggregation"
 
-__version__ = "$Revision: 90 $"
+__version__ = "$Revision: 151 $"
 
 import math
 import os
 import sys
 
 from collections import defaultdict
+from functools import partial
+from operator import itemgetter
 from rpy2.robjects import r, numpy2ri
 
 # XXX: Do this without the kludgy constants
-from .common import die, fill_array, get_ordered_labels, image_saver, key_by_field, load_genome, load_gff_data, load_segmentation, make_tabfilename, map_mnemonics, map_segments, r_source, SEGMENT_START_COL, SEGMENT_END_COL, setup_directory, tab_saver
+from .common import die, fill_array, image_saver, load_gff_data, load_segmentation, make_tabfilename, map_mnemonics, map_segments, maybe_gzip_open, r_source, SEGMENT_START_COL, SEGMENT_END_COL, setup_directory, tab_saver
 
 from .html import list2html, save_html_div
 
@@ -50,15 +52,15 @@ FLANK_5P = "5' flanking: %d bp"
 FLANK_3P = "3' flanking: %d bp"
 FLANK_COMPONENTS = [FLANK_5P, FLANK_3P]
 POINT_COMPONENTS = list(FLANK_COMPONENTS)
-REGION_COMPONENT = "internal: %d bp"
+REGION_COMPONENT = "internal"
 REGION_COMPONENTS = [FLANK_5P, REGION_COMPONENT, FLANK_3P]
 SPLICE_COMPONENTS = [
     "initial exon",
     "initial intron",
     "internal exons",
     "internal introns",
-    "terminal intron",
-    "terminal exon"]
+    "terminal exon",
+    "terminal intron"]
 CODING_COMPONENTS = [
     "initial 5' UTR",
     "5' UTR introns",
@@ -77,11 +79,10 @@ GENE_COMPONENTS = EXON_COMPONENTS + CODING_COMPONENTS
 EXON_COMPONENT = "exon"
 CDS_COMPONENT = "CDS"
 MIN_EXONS = 1
-MIN_CDSS = 1  # Must have at least 1 coding exon
-
+MIN_CDSS = 0
 
 # Default parameter values
-FLANK_BINS = 500  
+FLANK_BINS = 500
 REGION_BINS = 50
 INTRON_BINS = 50
 EXON_BINS = 25
@@ -108,7 +109,7 @@ def get_feature_factor(feature, mode):
         return feature["source"]
     else:
         return feature["name"]
-        
+
 
 ## Returns a dict of component -> window (list of base indices)
 def calc_feature_windows(feature, mode, components, component_bins):
@@ -118,7 +119,7 @@ def calc_feature_windows(feature, mode, components, component_bins):
     strand = feature["strand"]
     length = end - start
     assert length > 0
-    
+
     # Include flanks by default
     num_5p_bins = component_bins[FLANK_5P]
     num_3p_bins = component_bins[FLANK_3P]
@@ -140,9 +141,10 @@ def calc_feature_windows(feature, mode, components, component_bins):
         num_internal_bins = component_bins[component]
     except KeyError:
         num_internal_bins = 0
-        
+
     if num_internal_bins > length:
-        #print >> sys.stderr, "Warning: %d %s bins > %d bases" % (num_internal_bins, component, length)
+        #print >> sys.stderr, "Warning: %d %s bins > %d bases" % \
+        #    (num_internal_bins, component, length)
         num_internal_bins = 0
 
     # Calculate internal bin locations
@@ -151,7 +153,7 @@ def calc_feature_windows(feature, mode, components, component_bins):
     for i in range(0, num_internal_bins):
         # Round to nearest bin
         # XXX: This probably isn't quite right in every case, but it's very close
-        internal_bins.append(int(start + ((i + 1)*spacing) + 0.5)) 
+        internal_bins.append(int(start + ((i + 1)*spacing) + 0.5))
 
     bins_5p = []
     bins_3p = []
@@ -165,7 +167,7 @@ def calc_feature_windows(feature, mode, components, component_bins):
             bins_5p.append(start - num_5p_bins + i)
         for i in range(0, num_3p_bins):
             bins_3p.append(end + i)
-            
+
     if strand == "-":  # Flip for negative strand
         bins_5p = reversed(bins_5p)
         internal_bins = reversed(internal_bins)
@@ -182,7 +184,7 @@ def calc_feature_windows(feature, mode, components, component_bins):
     #for key, val in locals().iteritems():
     #    print >>sys.stderr, "%s : %s" % (key, val)
     #sys.exit(1)
-    
+
     return windows
 
 # Returns a dict: component -> number of bins for that component
@@ -197,18 +199,20 @@ def get_component_bins(components=[], flank_bins=FLANK_BINS,
         elif component in GENE_COMPONENTS:
             if "intron" in component:
                 bins = intron_bins
-            elif "exon" in component or "UTR" in component or "CDS" in component:
+            elif "exon" in component or \
+                    "UTR" in component or \
+                    "CDS" in component:
                 # All UTR components that don't contain "intron" are exons
                 bins = exon_bins
             else:
                 bins = region_bins
         else:
             bins = region_bins
-            
+
         component_bins[component] = bins
-        
+
     return component_bins
-    
+
 
 ## Given a list of features, returns:
 ##   A list of the values that occur for that field
@@ -226,6 +230,16 @@ def feature_field_values(features, field="name"):
 
 
 def preprocess_entries(entries):
+    """Convert list of gene feature entries to processed gene data.
+
+    If multiple transcripts are found, only the longest is taken.
+
+    :param entries: each entry is (chrom, start, end, strand, component, source)
+                    and all entries should correspond to the same gene_id
+    :type entries: list of 6-tuples
+    :returns: chrom, strand, source, list(exons), list(cds's)
+
+    """
     # Remove list of gene features and preprocess
     #   (extracting constant strand, chrom, source)
     # Returns gene info and list of exons and CDSs
@@ -234,7 +248,7 @@ def preprocess_entries(entries):
     gene_source = None
     exons = []
     cdss = []
-    
+
     for entry in entries:
         #print entry[4], "\t", entry[1:3], entry[3]
         chrom, start, end, strand, component, source = entry
@@ -243,19 +257,20 @@ def preprocess_entries(entries):
         if gene_strand is None:
             gene_strand = strand
         elif gene_strand != strand:
-            die("Found gene features on more than one strand: [%s, %s]" %
-                (gene_strand, strand))
+            die("Found gene features on more than one strand: [%s, %s]\n%s" %
+                (gene_strand, strand, str(entries)))
         if gene_chrom is None:
             gene_chrom = chrom
         elif gene_chrom != chrom:
-            die("Found gene features on more than one chromosome: [%s, %s]" %
-                (gene_chrom, chrom))
+            die("Found gene features on more than one chromosome:"
+                " [%s, %s]\n%s" % (gene_chrom, chrom, str(entries)))
         if gene_source is None:
             gene_source = source
         elif gene_source != source:
-            die("Found gene features from more than one source: [%s, %s]" %
-                (gene_source, source))
-            
+            print >>sys.stderr, ("Found gene features from more than one"
+                                 "source: [%s, %s]\n%s" %
+                                 (gene_source, source, str(entries)))
+
         partial_entry = (start, end)
         if component == EXON_COMPONENT:
             exons.append(partial_entry)
@@ -263,22 +278,24 @@ def preprocess_entries(entries):
             cdss.append(partial_entry)
     return gene_chrom, gene_strand, gene_source, exons, cdss
 
-def interpret_features_as_gene(entries):
+def interpret_features_as_gene(entries, min_exons=MIN_EXONS, min_cdss=MIN_CDSS):
     """Returns a set of gene-model entries based upon the given entries.
 
     Interprets the given set of entries in terms of an idealized gene model
     with 5' and 3' UTRs, initial, internal, and terminal exons and introns.
 
-    Returns a new set of entries based upon this gene model
-    or None if the entries don't fit the model.
+    :param entries: see preprocess_entries for a description
+    :returns: a new set of entries based upon this gene model
+              or None if the entries don't fit the model.
     """
 
-    gene_chrom, gene_strand, gene_source, exons, cdss = preprocess_entries(entries)
-            
-    # ... removing gene if without a start_codon feature or enough exons
-    if len(exons) < MIN_EXONS or len(cdss) < MIN_CDSS:
+    gene_chrom, gene_strand, gene_source, exons, cdss = \
+        preprocess_entries(entries)
+
+    # ... removing gene if without a coding exon or enough exons
+    if len(exons) < min_exons or len(cdss) < min_cdss:
         return None
-    
+
     exons.sort()
     cdss.sort()
 
@@ -289,14 +306,12 @@ def interpret_features_as_gene(entries):
         if prev_end is not None:
             introns.append((prev_end, exon_start))
         prev_end = exon_end
-    
+
     # Flip directions for '-' strand
     if gene_strand == "-":
         exons.reverse()
         introns.reverse()
         cdss.reverse()
-        
-    assert MIN_CDSS >= 1
 
     features = []
     # Add gene features to new list, renaming components
@@ -321,11 +336,16 @@ def interpret_features_as_gene(entries):
         features.append(make_feature(SPLICE_COMPONENTS[3], *intron))
     # last intron
     if len(introns) > 0:
-        features.append(make_feature(SPLICE_COMPONENTS[4], *introns[-1]))
+        features.append(make_feature(SPLICE_COMPONENTS[5], *introns[-1]))
     # last exon
-    features.append(make_feature(SPLICE_COMPONENTS[5], *exons[-1]))
+    features.append(make_feature(SPLICE_COMPONENTS[4], *exons[-1]))
 
 
+    # If there were no CDS's, then there is no point in dealing with UTR/CDS
+    #   regions, so skip all that.
+    if len(cdss) == 0:
+        return features
+
     # Binds local variable gene_strand to strand-correct
     def upstream(x, y):
         """Returns true if x starts before (is 5') of y, false otherwise"""
@@ -364,7 +384,7 @@ def interpret_features_as_gene(entries):
         # Add if exon was in UTR and has positive length
         if utr_list is not None and exon_start < exon_end:
             utr_list.append((exon_start, exon_end))  # Append tuple
-        
+
     for intron in introns:
         if upstream(intron, first_cds):
             UTR5p_introns.append(intron)
@@ -402,11 +422,53 @@ def interpret_features_as_gene(entries):
     # terminal 3' UTR
     if len(UTR3p_exons) > 0:
         features.append(make_feature(CODING_COMPONENTS[9], *UTR3p_exons[-1]))
-       
+
     return features
 
-def load_gtf_data(gtf_filename):
-    """Loads the gtf file in terms of idealized gene features.
+def get_transcript_length(entries):
+    """Returns the length of the transcript.
+
+    Assumes entries are all for the same transcript
+    
+    :param entries: each entry is (chrom, start, end, strand, component, source)
+                    and all entries should correspond to the same gene_id
+    :type entries: list of 6-tuples
+    :rtype: integer or None
+    
+    """
+    min_start = None
+    max_end = None
+    for entry in entries:
+        (chrom, start, end, strand, component, source) = entry
+        if component == EXON_COMPONENT:
+            if min_start is None or start < min_start:
+                min_start = start
+            if max_end is None or end > max_end:
+                max_end = end
+
+    if min_start is None or max_end is None:
+        return None
+    else:
+        return max_end - min_start
+
+def get_longest_transcript(transcript_dict):
+    """Return the longest transcript in the dict.
+
+    :type transcript_dict: transcript_id -> entries
+    :returns: tuple of (transcript_id, entries) for the longest transcript
+              or None if there were no transcripts.
+    """
+    max_length = 0
+    longest = None
+    for id, entries in transcript_dict.iteritems():
+        length = get_transcript_length(entries)
+        if length > max_length:
+            max_length = length
+            longest = (id, entries)
+    return longest
+
+def load_gtf_data(gtf_filename, min_exons=MIN_EXONS, min_cdss=MIN_CDSS):
+    """Load the gtf file in terms of idealized gene features.
 
     Parses the given feature file and replaces feature names from:
       CDS
@@ -430,53 +492,89 @@ def load_gtf_data(gtf_filename):
     exon or do not contain a start codon) are skipped.
     """
     # Start with establishing a dict:
-    #   geneid -> (chrom, start, end, strand, component)
+    #   gene_id -> dict(transcript_id -> (chrom, start, end, strand, component))
     #   start is 0-indexed, end is non-inclusive (BED)
-    genedict = defaultdict(list)
-    geneid_col = 0
-    with open(gtf_filename) as ifp:
+    gene_dict = defaultdict(partial(defaultdict, list))
+    gene_id_col = 0
+    transcript_id_col = 1
+    with maybe_gzip_open(gtf_filename) as ifp:
         for line in ifp:
+            # Ignore comment lines
+            if line.startswith("#"): continue
+
             # Parse tokens from GTF line
-            tokens = line.strip().split("\t")
-            chrom = tokens[0]
-            source = tokens[1]
+            try:
+                tokens = line.strip().split("\t")
+                chrom = tokens[0]
+                source = tokens[1]
+            except ValueError:
+                die("Error parsing chrom (field 1) or source (field 2)"
+                    " from GTF line: %s" % line)
             try:
                 start = int(tokens[3]) - 1
                 end = int(tokens[4])
             except ValueError:
-                die("Error parsing start (field 4) or end (field 5) from GTF line: %s" % line)
+                die("Error parsing start (field 4) or end (field 5)"
+                    " from GTF line: %s" % line)
             strand = tokens[6]
             if strand != "+" and strand != "-":
-                die("Expected +/- strand, but found: %s in GTF line: %s" % (strand, line))
+                die("Expected +/- strand, but found: %s in GTF line:"
+                    " %s" % (strand, line))
             component = tokens[2]
-                
-            properties = tokens[8].split(";")
-            if not properties[geneid_col].startswith("gene_id"):
-                die("Expected gene_id to be first property in field 9 of GTF line: %s" % line)
-            
-            geneid = properties[geneid_col].split()[1].strip('"')
+
+            properties = tokens[8].split("; ")
+            if not properties[gene_id_col].startswith("gene_id"):
+                # Try to find the gene_id column
+                gene_id_col = None
+                for col_offset, prop in enumerate(properties):
+                    if prop.startswith("gene_id"):
+                        gene_id_col = col_offset
+
+                if gene_id_col is None:
+                    die("Could not find the gene_id column in GTF line: %s" %
+                        line)
+            gene_id = properties[gene_id_col].split()[1].strip('"')
+
+            if not properties[transcript_id_col].startswith("transcript_id"):
+                # Try to find the transcript_id column
+                transcript_id_col = None
+                for col_offset, prop in enumerate(properties):
+                    if prop.startswith("transcript_id"):
+                        transcript_id_col = col_offset
+
+                if transcript_id_col is None:
+                    die("Could not find the transcript_id column in"
+                        " GTF line: %s" % line)
+            transcript_id = properties[transcript_id_col].split()[1].strip('"')
 
             # Add feature to dict
             entry = (chrom, start, end, strand, component, source)
-            genedict[geneid].append(entry)
+            gene_dict[gene_id][transcript_id].append(entry)
 
     # Eventually create feature dict: chrom -> list(features)
     data = defaultdict(list)
-    for geneid, entries in genedict.iteritems():
-        entries = interpret_features_as_gene(entries)
-        if entries is not None:
+    for gene_id, transcript_dict in gene_dict.iteritems():
+
+        # Select only longest transcript
+        transcript_id, transcript = get_longest_transcript(transcript_dict)
+        gene_model = interpret_features_as_gene(transcript,
+                                                min_exons=min_exons,
+                                                min_cdss=min_cdss)
+
+        if gene_model is not None:
             # Add features to a normal, chrom-based feature dict
-            for entry in entries:
-                #print entry[4], "\t", entry[1:3], entry[3]
-                chrom, start, end, strand, component, source=entry
+            for gene_part in gene_model:
+#                 print str(gene_part)
+                chrom, start, end, strand, component, source = gene_part
                 feature = dict(start=start, end=end, strand=strand,
                                name=component, source=source)
                 data[chrom].append(feature)
-            #raw_input()
+                
+#         raw_input()
 
     # Sort features by ascending start        
     for chrom_features in data.itervalues():
-        chrom_features.sort(key=key_by_field("start"))
+        chrom_features.sort(key=itemgetter("start"))
         
     return data
     
@@ -490,12 +588,14 @@ def load_gtf_data(gtf_filename):
 ##     flanking regions before the 1st and after the last component in the list
 ##     Each feature's component entry must match one of these exactly
 ## component_bins is a dict: component -> number of bins for that component
+## If nofactor: all factors found are treated as only factor in factors
 ## Returns:
 ##   factors: a list of the factors (groups) aggregated over
 ##   counts: dict(label_key -> dict(feature -> dict(component -> histogram)))
-##   component_counts: an array of the number of factors with valid labels at each bin pos
+##   component_counts: an array of the number of factors with valid
+##     labels at each bin pos
 def calc_aggregation(segmentation, mode, features, factors, components=[],
-                     component_bins=None, quick=False):
+                     component_bins=None, quick=False, nofactor=False):
     assert len(factors) > 0
 
     if component_bins is None:
@@ -503,13 +603,22 @@ def calc_aggregation(segmentation, mode, features, factors, components=[],
     else:
         for component in components:
             assert component in component_bins
+
+    if nofactor:
+        assert len(factors) == 1
             
     labels = segmentation.labels
 
     print >>sys.stderr, "\tFactors: %s" % factors
     print >>sys.stderr, "\tComponents and bins:"
     for component in components:
-        print >>sys.stderr, "\t\t%s: %d" % (component, component_bins[component])
+        print >>sys.stderr, "\t\t",
+        # Try to substitute number of bins in first
+        try:
+            print >>sys.stderr, component % component_bins[component]
+        except TypeError:
+            print >>sys.stderr, "%s: %d" % (component,
+                                            component_bins[component])
 
     # Number of features aggregated over at each position for each factor
     # and component
@@ -565,7 +674,10 @@ def calc_aggregation(segmentation, mode, features, factors, components=[],
                     feature["start"] > segmentation_end:
                 continue  # Feature outside segmentation
 
-            factor = get_feature_factor(feature, mode)
+            if nofactor:
+                factor = factors[0]
+            else:
+                factor = get_feature_factor(feature, mode)
 
             # Spread internal bins throughout feature
             component_windows = calc_feature_windows(feature, mode, components,
@@ -621,7 +733,7 @@ def save_tab(labels, counts, components, component_bins,
                             component_name = component % component_bins[component]
                         except TypeError:
                             component_name = component
-                            
+
                         row = make_row(labels[label_key],
                                        factor,
                                        component_name,
@@ -654,7 +766,7 @@ def save_plot(dirpath, num_labels, mode, spacers=len(EXON_COMPONENTS),
                                      mnemonics=mnemonics,
                                      genemode=(mode == GENE_MODE),
                                      spacers=spacers))
-        
+
 #     with image_saver(dirpath, "%s.label_normalized" % NAMEBASE,
 #                      height=png_size,
 #                      width=png_size,
@@ -663,7 +775,7 @@ def save_plot(dirpath, num_labels, mode, spacers=len(EXON_COMPONENTS),
 #                                      normalize_labels=True,
 #                                      mnemonics=mnemonics,
 #                                      genemode=(mode == GENE_MODE)))
-    
+
 
 def save_html(dirpath, featurefilename, mode, num_features, factors,
               components, clobber=False):
@@ -674,44 +786,53 @@ def save_html(dirpath, featurefilename, mode, num_features, factors,
                   module=MODULE, featurefilename=featurebasename,
                   numfeatures=num_features, mode=mode, factorlist=factorlist,
                   title=title)
-    
+
 def print_array(arr, tag="", type="%d"):
     fstring = "%%s:  %s, %s, ..., %s, ..., %s, %s" % tuple([type]*5)
-    print fstring % (tag,
-                     arr[0],
-                     arr[1],
-                     arr[int(arr.shape[0] / 2)],
-                     arr[-2],
-                     arr[-1])
-    
+    print >>sys.stderr, fstring % (tag,
+                                   arr[0],
+                                   arr[1],
+                                   arr[int(arr.shape[0] / 2)],
+                                   arr[-2],
+                                   arr[-1])
+
 ## Package entry point
 def validate(bedfilename, featurefilename, dirpath,
              flank_bins=FLANK_BINS, region_bins=REGION_BINS,
              intron_bins=INTRON_BINS, exon_bins=EXON_BINS,
-             mode=DEFAULT_MODE, clobber=False, quick=False,
-             replot=False, noplot=False, mnemonicfilename=None):
+             nofactor=False, mode=DEFAULT_MODE, clobber=False,
+             quick=False, replot=False, noplot=False,
+             min_exons=MIN_EXONS, min_cdss=MIN_CDSS,
+             mnemonicfilename=None):
     setup_directory(dirpath)
     segmentation = load_segmentation(bedfilename)
-    
+
     assert segmentation is not None
 
     print >>sys.stderr, "Using file %s" % featurefilename
     if mode == "gene":
-        load_func = load_gtf_data
+        features = load_gtf_data(featurefilename,
+                                 min_exons=min_exons,
+                                 min_cdss=min_cdss)
     else:
-        load_func = load_gff_data
-    features = load_func(featurefilename)
+        features = load_gff_data(featurefilename)
     assert features is not None
-        
+
+    if nofactor:
+        factors = ["factor"]
+
     if mode == GENE_MODE:
-        factors = feature_field_values(features, "source")
         components = GENE_COMPONENTS
+        if not nofactor:
+            factors = feature_field_values(features, "source")
     elif mode == REGION_MODE:
-        factors = feature_field_values(features, "name")
         components = REGION_COMPONENTS
+        if not nofactor:
+            factors = feature_field_values(features, "name")
     elif mode == POINT_MODE:
-        factors = feature_field_values(features, "name")
         components = POINT_COMPONENTS
+        if not nofactor:
+            factors = feature_field_values(features, "name")
 
     labels = segmentation.labels
     mnemonics = map_mnemonics(labels, mnemonicfilename)
@@ -722,21 +843,22 @@ def validate(bedfilename, featurefilename, dirpath,
     print >>sys.stderr, "\tAggregating over %d features" % num_features
 
     if not replot:
-        component_bins = get_component_bins(components, 
+        component_bins = get_component_bins(components,
                                             flank_bins=flank_bins,
                                             region_bins=region_bins,
                                             intron_bins=intron_bins,
                                             exon_bins=exon_bins)
-        
+
         res = calc_aggregation(segmentation, mode=mode, features=features,
                                factors=factors, components=components,
-                               component_bins=component_bins, quick=quick)
+                               component_bins=component_bins, quick=quick,
+                               nofactor=nofactor)
         counts, label_counts, component_counts, counted_features = res
         for factor in factors:
             for component in components:
                 print_array(component_counts[factor][component],
                             tag=factor+":"+component+" counts")
-            
+
         save_tab(labels, counts, components, component_bins,
                  component_counts, label_counts,
                  counted_features, dirpath, clobber=clobber)
@@ -748,7 +870,7 @@ def validate(bedfilename, featurefilename, dirpath,
     save_html(dirpath, featurefilename, mode=mode, factors=factors,
               components=components, num_features=num_features,
               clobber=clobber)
-    
+
 def parse_options(args):
     from optparse import OptionParser, OptionGroup
 
@@ -763,7 +885,7 @@ def parse_options(args):
                       default=None,
                       help="If specified, labels will be shown using"
                       " mnemonics found in this file")
-    group.add_option("-o", "--outdir", 
+    group.add_option("-o", "--outdir",
                       dest="outdir", default="%s" % MODULE,
                       help="File output directory (will be created"
                       " if it does not exist) [default: %default]")
@@ -771,23 +893,26 @@ def parse_options(args):
 
     group = OptionGroup(parser, "Flags")
     group.add_option("--clobber", action="store_true",
-                      dest="clobber", default=False,
-                      help="Overwrite existing output files if the specified"
-                      " directory already exists.")
-    group.add_option("--quick", action="store_true", 
-                      dest="quick", default=False,
-                      help="Compute values only for one chromosome.")
-    group.add_option("--replot", action="store_true", 
-                      dest="replot", default=False,
-                      help="Load data from output tab files and"
-                      " regenerate plots instead of recomputing data")
-    group.add_option("--noplot", action="store_true", 
-                      dest="noplot", default=False,
-                      help="Do not generate plots")
+                     dest="clobber", default=False,
+                     help="Overwrite existing output files if the specified"
+                     " directory already exists.")
+    group.add_option("--quick", action="store_true",
+                     dest="quick", default=False,
+                     help="Compute values only for one chromosome.")
+    group.add_option("--replot", action="store_true",
+                     dest="replot", default=False,
+                     help="Load data from output tab files and"
+                     " regenerate plots instead of recomputing data")
+    group.add_option("--noplot", action="store_true",
+                     dest="noplot", default=False,
+                     help="Do not generate plots")
+    group.add_option("--no-factor", action="store_true",
+                     dest="nofactor", default=False,
+                     help="Do not separate data into different factors")
     parser.add_option_group(group)
 
-    group = OptionGroup(parser, "Aggregation options")
-    group.add_option("-m", "--mode", choices=MODES, 
+    group = OptionGroup(parser, "Main aggregation options")
+    group.add_option("-m", "--mode", choices=MODES,
                      dest="mode", type="choice", default=DEFAULT_MODE,
                      help="one of: "+str(MODES)+", --gene not implemented"
                      " [default: %default]")
@@ -795,23 +920,34 @@ def parse_options(args):
                      dest="flankbins", type="int", default=FLANK_BINS,
                      help="Aggregate this many base pairs off each"
                      " end of feature or gene [default: %default]")
-    group.add_option("-r", "--region-bins", type="int", 
+    group.add_option("-r", "--region-bins", type="int",
                      dest="regionbins", default=REGION_BINS,
                      help="If --mode=region, aggregate over each internal"
                      "feature using this many evenly-spaced bins"
                      " [default: %default]")
-    group.add_option("-i", "--intron-bins", type="int", 
-                     dest="intronbins", default=INTRON_BINS,
-                     help="If --mode=gene, Aggregate over each intron"
-                     "using this many evenly-spaced bins"
-                     " [default: %default]")
-    group.add_option("-e", "--exon-bins", type="int", 
-                     dest="exonbins", default=EXON_BINS,
-                     help="If --mode=gene, Aggregate over each exon"
-                     "using this many evenly-spaced bins"
-                     " [default: %default]")
     parser.add_option_group(group)
-    
+
+    group = OptionGroup(parser, "Gene aggregation options")
+    group.add_option("-i", "--intron-bins", type="int",
+                     dest="intronbins", default=INTRON_BINS,
+                     help="Aggregate over each intron"
+                     "using this many evenly-spaced bins"
+                     " [default: %default]")
+    group.add_option("-e", "--exon-bins", type="int",
+                     dest="exonbins", default=EXON_BINS,
+                     help="Aggregate over each exon"
+                     "using this many evenly-spaced bins"
+                     " [default: %default]")
+    group.add_option("--min-exons", type="int",
+                     dest="min_exons", default=MIN_EXONS,
+                     help="Only consider genes with at least this many exons"
+                     " [default: %default]")
+    group.add_option("--min-coding-exons", type="int",
+                     dest="min_cdss", default=MIN_CDSS,
+                     help="Only consider genes with at least this many coding"
+                     " exon [default: %default]")
+    parser.add_option_group(group)
+
     (options, args) = parser.parse_args(args)
 
     if len(args) < 2:
@@ -819,22 +955,23 @@ def parse_options(args):
 
     if options.noplot and options.replot:
         parser.error("noplot and replot are contradictory")
-        
+
     return (options, args)
-    
+
 ## Command-line entry point
 def main(args=sys.argv[1:]):
     (options, args) = parse_options(args)
     bedfilename = args[0]
     featurefilename = args[1]
-    
+
     validate(bedfilename, featurefilename, options.outdir,
              flank_bins=options.flankbins, region_bins=options.regionbins,
              intron_bins=options.intronbins, exon_bins=options.exonbins,
              clobber=options.clobber, quick=options.quick,
              replot=options.replot, noplot=options.noplot,
-             mode=options.mode,
+             nofactor=options.nofactor, mode=options.mode,
+             min_exons=options.min_exons, min_cdss=options.min_cdss,
              mnemonicfilename=options.mnemonicfilename)
-        
+
 if __name__ == "__main__":
     sys.exit(main())
