@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 from __future__ import division, with_statement
 
-__version__ = "$Revision: 90 $"
+__version__ = "$Revision: 151 $"
 
 """
 label_transition.py:
@@ -33,13 +33,12 @@ import os
 import sys
 
 
-from contextlib import contextmanager
-from functools import partial
-from numpy import array, empty, sort, where, zeros
+from numpy import array, empty, where, zeros
+from subprocess import call
 from rpy2.robjects import r, rinterface, numpy2ri
 
 # XXX: Do this without the kludgy constants
-from .common import check_clobber, die, get_ordered_labels, image_saver, load_segmentation, load_genome, make_dotfilename, make_filename, make_pdffilename, make_pngfilename, make_namebase_summary, make_tabfilename, map_mnemonics, OutputMasker, r_source, SEGMENT_START_COL, SEGMENT_END_COL, SEGMENT_LABEL_KEY_COL, setup_directory, tab_saver
+from .common import check_clobber, die, get_ordered_labels, image_saver, load_segmentation, make_dotfilename, make_pdffilename, make_pngfilename, make_namebase_summary, make_tabfilename, map_mnemonics, r_source, SEGMENT_START_COL, SEGMENT_END_COL, SEGMENT_LABEL_KEY_COL, setup_directory, tab_saver
 
 from .html import save_html_div
 
@@ -76,7 +75,7 @@ def calc_transitions(segmentation):
     counts = zeros((len(labels), len(labels)), dtype="int")
     for segments in segmentation.chromosomes.itervalues():
         # Inch along rows, looking at all pairs
-	row_it = iter(segments)
+        row_it = iter(segments)
         row1 = None
         for row2 in row_it:
             if row1 is not None:  # Skip first loop
@@ -87,14 +86,19 @@ def calc_transitions(segmentation):
                     counts[label_key1, label_key2] += 1
                     if label_key1 == label_key2:
                         if self_transition_count == 0:
-                            print >>sys.stderr, "WARNING: unexpected self-transition %s -> %s\n\tRow 1: %s\n\tRow 2: %s" % (labels[label_key1], labels[label_key2], row1, row2)
+                            print >>sys.stderr, \
+                                ("WARNING: unexpected self-transition"
+                                 "%s -> %s\n\tRow 1: %s\n\tRow 2: %s" % \
+                                (labels[label_key1], labels[label_key2],
+                                 row1, row2))
                         self_transition_count += 1
 
             row1 = row2  # Inch forward
-            
+
     if self_transition_count > 0:
-        print >>sys.stderr, "WARNING: %d self-transitions observed" % self_transition_count
-        
+        print >>sys.stderr, "WARNING: %d self-transitions observed" % \
+            self_transition_count
+
     # Row-normalize to turn counts into probabilities
     probs = empty(counts.shape, dtype='double')
     for i, row in enumerate(counts):
@@ -105,7 +109,7 @@ def calc_transitions(segmentation):
 
 def save_tab(labels, probs, dirpath, clobber=False):
     ordered_keys, labels = get_ordered_labels(labels)
-    
+
     # Get fieldnames in order
     fieldnames = list(labels[key] for key in ordered_keys)
     with tab_saver(dirpath, NAMEBASE, fieldnames,
@@ -134,7 +138,7 @@ def save_summary_tab(labels, probs, dirpath, mnemonics=None, clobber=False):
 
 def save_html(dirpath, p_thresh, q_thresh, clobber=False):
     extra_namebases = {"graph": NAMEBASE_GRAPH}
-    
+
     if p_thresh > 0:
         thresh = "P > %s" % p_thresh
     elif q_thresh > 0:
@@ -151,7 +155,7 @@ def save_plot(dirpath, ddgram=False, clobber=False, mnemonics=[],
     tabfilename = make_tabfilename(dirpath, NAMEBASE)
     if not os.path.isfile(tabfilename):
         die("Unable to find tab file: %s" % tabfilename)
-        
+
     with image_saver(dirpath, NAMEBASE,
                      height=PNG_SIZE,
                      width=PNG_SIZE,
@@ -166,9 +170,9 @@ def plot_gmtk_transitions(filename, dirpath, clobber=False, verbose=False,
     """Plots transition probabilities from gmtk params file.
     """
     assert filename is not None and os.path.isfile(filename)
-    
+
     start_R()
-    
+
     # Plot transition probabilies
     with image_saver(dirpath, NAMEBASE,
                      height=PNG_SIZE,
@@ -188,14 +192,16 @@ def plot_gmtk_transitions(filename, dirpath, clobber=False, verbose=False,
                                      mnemonics=mnemonics))
 
 def save_graph(labels, probs, dirpath, q_thresh=Q_THRESH, p_thresh=P_THRESH,
-               clobber=False, mnemonics=[]):
+               clobber=False, mnemonics=[], fontname="Helvetica",
+               lenient_thresh=True, gene_graph=False):
     assert labels is not None and probs is not None
     try:
         import pygraphviz as pgv
     except ImportError:
-        print >>sys.stderr, "Unable to load PyGraphviz library. Skipping transition graph"
+        print >>sys.stderr, ("Unable to load PyGraphviz library."
+                             " Skipping transition graph")
         return
-    
+
     dotfilename = make_dotfilename(dirpath, NAMEBASE_GRAPH)
     check_clobber(dotfilename, clobber)
     pngfilename = make_pngfilename(dirpath, NAMEBASE_GRAPH)
@@ -205,21 +211,27 @@ def save_graph(labels, probs, dirpath, q_thresh=Q_THRESH, p_thresh=P_THRESH,
 
     # Replace labels with mnemonic labels, if mnemonics are given
     ordered_keys, labels = get_ordered_labels(labels, mnemonics)
-    
-    # Threshold in order dependent upon flag
+
+    # Threshold
     if q_thresh > 0:
         quantile = r['matrix.find_quantile'](probs, q_thresh)[0]
         print >>sys.stderr, "Removing edges below %.4f" % float(quantile)
         probs[probs < quantile] = 0
     elif p_thresh > 0:
-        print >>sys.stderr, "Removing edges below %.4f" % p_thresh
-        probs[probs < p_thresh] = 0
-        
+        print >>sys.stderr, "Removing connections below %.4f" % p_thresh
+        if lenient_thresh:
+            row_wise_remove = (probs < p_thresh)
+            col_wise = probs / probs.sum(axis=0) # Col-normalize
+            col_wise_remove = (col_wise < p_thresh)
+            probs[row_wise_remove & col_wise_remove] = 0
+        else:
+            probs[probs < p_thresh] = 0
+
     # Create graph out of non-zero edges
     G = pgv.AGraph(strict=False, directed=True)
 
     rows, cols = where(probs > 0)
-        
+
     max_val = probs.max()
     min_val = probs[probs > 0].min()
     weights = (probs - min_val)/(max_val-min_val) + BASE_WEIGHT
@@ -230,26 +242,41 @@ def save_graph(labels, probs, dirpath, q_thresh=Q_THRESH, p_thresh=P_THRESH,
                    penwidth=str(EDGE_WEIGHT * weight),
                    arrowsize=str(ARROW_WEIGHT * weight))
 
+    if gene_graph:
+        G.node_attr.update(fontname=fontname, shape="plaintext")
+        ps_dir = os.path.join(os.path.split(dirpath)[0], "images")
+        for node in G.nodes():
+            node.attr["image"] = "%s/%s.ps" % (ps_dir, str(node))
+            node.attr["label"] = " "
+
     G.write(dotfilename)
 
-    print >>sys.stderr, "Drawing graph...",
-    # Mask obnoxious font errors
-    sys.stderr = OutputMasker(sys.stderr)
+    print >>sys.stderr, "Drawing graphs...",
     G.layout()
-    sys.stderr = sys.stderr.restore()
+
     try:
         G.draw(pngfilename)
     except:
         print >>sys.stderr, "Failed to draw png graph"
-        
+
     try:
-        G.draw(pdffilename)
-    except:
-        print >>sys.stderr, "Failed to draw pdf graph"
-        
+        # Try to generate pdf from dot
+        if gene_graph:
+            layout_prog = "dot"
+        else:
+            layout_prog = "neato"
+
+        cmd = " ".join([layout_prog, "-Tps2", dotfilename, "|",
+                        "ps2pdf", "-dAutoRotatePages=/None", "-", pdffilename])
+        code = call(cmd, shell=True)
+        if code != 0:
+            raise Exception()
+    except Exception, e:
+        print >>sys.stderr, "Failed to draw pdf graph: %s" % str(e)
+
     print >>sys.stderr, "done"
 
-def create_mnemonic_file(gmtk_file, dirpath, clobber=False, namebase = None):
+def create_mnemonic_file(gmtk_file, dirpath, clobber=False, namebase=None):
     """Generate a mnemonic file for GMTK params with R clustering code.
 
     Calls R code that writes mnemonics to a file.
@@ -265,7 +292,7 @@ def create_mnemonic_file(gmtk_file, dirpath, clobber=False, namebase = None):
             namebase = gmtk_filebase
 
     mnemonic_base = os.extsep.join([namebase, "mnemonics"])
-    
+
     filename = make_tabfilename(dirpath, mnemonic_base)
     check_clobber(filename, clobber)
 
@@ -277,16 +304,17 @@ def create_mnemonic_file(gmtk_file, dirpath, clobber=False, namebase = None):
         print >>sys.stderr, "Generated mnemonic file: %s" % mnemonicfilename
         return mnemonicfilename
     except rinterface.RRuntimeError:
-        print >>sys.stderr, "ERROR: Failed to create mnemonic file. Continuing without mnemonics."
+        print >>sys.stderr, ("ERROR: Failed to create mnemonic file."
+                             " Continuing without mnemonics.")
         return None
-    
+
 def get_num_gmtk_labels(gmtk_file):
     """Load number of labels from gmtk params file."""
     start_R()
-    
+
     num_labels = int(r["num.gmtk.labels"](gmtk_file)[0])
     return num_labels
-    
+
 def get_default_labels(num_labels):
     """Generate default labels (0 through num_labels-1)"""
     return dict([(val, str(val)) for val in range(0, num_labels)])
@@ -302,12 +330,12 @@ def load_gmtk_transitions(gmtk_file):
     # Rpy automatically transposes, so need to transpose it back
     probs = array(r_data, dtype="double").transpose()
     return probs
-    
+
 
 ## Package entry point
 def validate_gmtk(gmtk_file, dirpath, ddgram=False, p_thresh=P_THRESH,
                   q_thresh=Q_THRESH, noplot=False, nograph=False,
-                  clobber=False, mnemonicfilename=None):
+                  gene_graph=False, clobber=False, mnemonicfilename=None):
     setup_directory(dirpath)
 
     if not os.path.isfile(gmtk_file):
@@ -325,27 +353,28 @@ def validate_gmtk(gmtk_file, dirpath, ddgram=False, p_thresh=P_THRESH,
     if not noplot:
         plot_gmtk_transitions(gmtk_file, dirpath, mnemonics=mnemonics,
                               clobber=clobber)
-    
+
     if not nograph:
         probs = load_gmtk_transitions(gmtk_file)
         save_graph(labels, probs, dirpath, clobber=clobber,
-                   p_thresh=p_thresh, q_thresh=q_thresh, mnemonics=mnemonics)
+                   p_thresh=p_thresh, q_thresh=q_thresh,
+                   gene_graph=gene_graph, mnemonics=mnemonics)
 
     save_html(dirpath, p_thresh=p_thresh, q_thresh=q_thresh,
               clobber=clobber)
-    
+
 ## Package entry point
 def validate(bedfilename, dirpath, ddgram=False, p_thresh=P_THRESH,
              q_thresh=Q_THRESH, noplot=False, nograph=False,
-             clobber=False, mnemonicfilename=None):
+             gene_graph=False, clobber=False, mnemonicfilename=None):
     setup_directory(dirpath)
 
     segmentation = load_segmentation(bedfilename)
     assert segmentation is not None
-    
+
     # Calculate transition probabilities for each label
     labels, probs = calc_transitions(segmentation)
-        
+
     mnemonics = map_mnemonics(labels, mnemonicfilename)
     save_summary_tab(labels, probs, dirpath,
                      clobber=clobber, mnemonics=mnemonics)
@@ -353,10 +382,11 @@ def validate(bedfilename, dirpath, ddgram=False, p_thresh=P_THRESH,
     if not noplot:
         save_plot(dirpath, ddgram=ddgram,
                   clobber=clobber, mnemonics=mnemonics)
-            
+
     if not nograph:
         save_graph(labels, probs, dirpath, clobber=clobber,
-                   p_thresh=p_thresh, q_thresh=q_thresh, mnemonics=mnemonics)
+                   p_thresh=p_thresh, q_thresh=q_thresh,
+                   gene_graph=gene_graph, mnemonics=mnemonics)
 
     save_html(dirpath, p_thresh=p_thresh, q_thresh=q_thresh,
               clobber=clobber)
@@ -367,26 +397,26 @@ def parse_options(args):
     usage = "%prog [OPTIONS] BEDFILE"
     version = "%%prog %s" % __version__
     parser = OptionParser(usage=usage, version=version)
-    
+
     parser.add_option("--clobber", action="store_true",
                       dest="clobber", default=False,
                       help="Overwrite existing output files if the specified"
                       " directory already exists.")
-    parser.add_option("--noplot", action="store_true", 
+    parser.add_option("--noplot", action="store_true",
                       dest="noplot", default=False,
                       help="Do not generate transition plots")
-    parser.add_option("--nograph", action="store_true", 
+    parser.add_option("--nograph", action="store_true",
                       dest="nograph", default=False,
                       help="Do not generate transition graph")
     parser.add_option("--mnemonic-file", dest="mnemonic_file",
                       default=None,
                       help="If specified, labels will be shown using"
                       " mnemonics found in this file")
-    parser.add_option("-o", "--outdir", 
+    parser.add_option("-o", "--outdir",
                       dest="outdir", default="%s" % MODULE,
                       help="File output directory (will be created"
                       " if it does not exist) [default: %default]")
-    
+
     group = OptionGroup(parser, "Transition frequency plot options")
     group.add_option("--dd", "--dendrogram", dest="ddgram",
                      default=False, action="store_true",
@@ -403,15 +433,19 @@ def parse_options(args):
                      type="float", default=Q_THRESH,
                      help="ignore transitions with probabilities below this"
                      " probability quantile [default: %default]")
+    group.add_option("--gene-graph", dest="gene_graph",
+                     default=False, action="store_true",
+                     help="Make each node of the graph a reference to a .ps"
+                     " image an \"image\" subdirectory. Currently, these .ps"
+                     " files need to be made separately.")
     parser.add_option_group(group)
 
     group = OptionGroup(parser, "Non-segmentation files")
-    parser.add_option("--gmtk-params", dest="gmtk_file",
-                      default=None,
+    group.add_option("-g", "--gmtk-params", dest="gmtk_file", default=None,
                       help="If specified, parameters in the given GMTK"
                       " file will be used to generate plots instead of the"
                       " observed transitions in the BEDFILE. The BEDFILE"
-                      " will not be used")
+                      " will not be used and is not required.")
     parser.add_option_group(group)
 
     (options, args) = parser.parse_args(args)
@@ -425,9 +459,9 @@ def parse_options(args):
         parser.error("Quantile threshold should be in range [0, 1]")
     if options.p_thresh < 0 or options.p_thresh > 1:
         parser.error("Probability threshold should be in range [0, 1]")
-    
+
     return (options, args)
-    
+
 ## Command-line entry point
 def main(args=sys.argv[1:]):
     (options, args) = parse_options(args)
@@ -436,16 +470,16 @@ def main(args=sys.argv[1:]):
         validate_gmtk(options.gmtk_file, options.outdir,
                       p_thresh=options.p_thresh, q_thresh=options.q_thresh,
                       clobber=options.clobber, noplot=options.noplot,
-                      nograph=options.nograph,
+                      nograph=options.nograph, gene_graph=options.gene_graph,
                       mnemonicfilename=options.mnemonic_file)
-        
+
     else:
         bedfilename=args[0]
         validate(bedfilename, options.outdir, ddgram=options.ddgram,
                  p_thresh=options.p_thresh, q_thresh=options.q_thresh,
                  clobber=options.clobber, noplot=options.noplot,
-                 nograph=options.nograph,
+                 nograph=options.nograph, gene_graph=options.gene_graph,
                  mnemonicfilename=options.mnemonic_file)
-        
+
 if __name__ == "__main__":
     sys.exit(main())
