@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 from __future__ import division, with_statement
 
-__version__ = "$Revision: 90 $"
+__version__ = "$Revision: 151 $"
 
 """
 validate.py:
@@ -16,8 +16,8 @@ import os
 import sys
 
 from collections import defaultdict
-from numpy import array, zeros
-from rpy2.robjects import r, rinterface, numpy2ri
+from numpy import zeros
+from rpy2.robjects import r, numpy2ri
 
 # XXX: Do this without the kludgy constants
 from .common import die, get_ordered_labels, get_supercontig_splice, \
